from .common import Common
from .clickhouse import Clickhouse
import requests
from datetime import datetime,timedelta
import clickhouse_connect
import pandas as pd
import os
import math
from dateutil import parser
import time
import hashlib
from io import StringIO
import json
from dateutil.relativedelta import relativedelta


class ALFAbyDate:
    def __init__(self,  bot_token:str, chats:str, message_type: str, subd: str,
                 host: str, port: str, username: str, password: str, database: str,
                                  add_name: str, main_url:str, token: str ,  xappkey:str, email: str ,
                 start: str, backfill_days: int, reports :str, branches: str):
        self.bot_token = bot_token
        self.chat_list = chats.replace(' ', '').split(',')
        self.message_type = message_type
        self.common = Common(self.bot_token, self.chat_list, self.message_type)
        self.main_url = main_url
        self.token = token
        self.branches_list = branches.replace(' ','').strip().split(',')
        self.xappkey = xappkey
        self.email = email
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.database = database
        self.subd = subd
        self.add_name = self.common.transliterate_key(add_name)
        self.now = datetime.now()
        self.today = datetime.now().date()
        self.start = start
        self.reports = reports
        self.backfill_days = backfill_days
        self.platform = 'alfa'
        self.err429 = False
        self.source_dict = {
            'branch': {
                'platform': 'alfa',
                'report_name': 'branch',
                'upload_table': 'branch',
                'func_name': self.get_branch,
                'uniq_columns': 'id',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': False,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 1
            },
            'location': {
                'platform': 'alfa',
                'report_name': 'location',
                'upload_table': 'location',
                'func_name': self.get_location,
                'uniq_columns': 'id',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': False,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 1
            },
            'task_null': {
                'platform': 'alfa',
                'report_name': 'task_null',
                'upload_table': 'task',
                'func_name': self.get_task_null,
                'uniq_columns': 'id',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': False,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 1
            },
            'tariff': {
                'platform': 'alfa',
                'report_name': 'tariff',
                'upload_table': 'tariff',
                'func_name': self.get_tariff,
                'uniq_columns': 'id',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': False,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 1
            },
            'teacher': {
                'platform': 'alfa',
                'report_name': 'teacher',
                'upload_table': 'teacher',
                'func_name': self.get_teacher,
                'uniq_columns': 'id',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'delete_all',
                'history': False,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 1
            },
            'subject': {
                'platform': 'alfa',
                'report_name': 'subject',
                'upload_table': 'subject',
                'func_name': self.get_subject,
                'uniq_columns': 'id',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'delete_all',
                'history': False,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 1
            },
            'study_status': {
                'platform': 'alfa',
                'report_name': 'study_status',
                'upload_table': 'study_status',
                'func_name': self.get_study_status,
                'uniq_columns': 'id',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'delete_all',
                'history': False,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 1
            },
            'discount': {
                'platform': 'alfa',
                'report_name': 'discount',
                'upload_table': 'discount',
                'func_name': self.get_discount,
                'uniq_columns': 'id',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'delete_all',
                'history': False,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 1
            },
            'lead_status': {
                'platform': 'alfa',
                'report_name': 'lead_status',
                'upload_table': 'lead_status',
                'func_name': self.get_lead_status,
                'uniq_columns': 'id',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'delete_all',
                'history': False,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 1
            },
            'lead_source': {
                'platform': 'alfa',
                'report_name': 'lead_source',
                'upload_table': 'lead_source',
                'func_name': self.get_lead_source,
                'uniq_columns': 'id',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'delete_all',
                'history': False,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 1
            },
            'customer': {
                'platform': 'alfa',
                'report_name': 'customer',
                'upload_table': 'customer',
                'func_name': self.get_customer,
                'uniq_columns': 'id',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': True,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 1
            },
            'regular_lesson': {
                'platform': 'alfa',
                'report_name': 'regular_lesson',
                'upload_table': 'regular_lesson',
                'func_name': self.get_regular_lesson,
                'uniq_columns': 'id',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': True,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 1
            },
            'group': {
                'platform': 'alfa',
                'report_name': 'group',
                'upload_table': 'group',
                'func_name': self.get_group,
                'uniq_columns': 'id',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': True,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 1
            },
            'pay': {
                'platform': 'alfa',
                'report_name': 'pay',
                'upload_table': 'pay',
                'func_name': self.get_pay,
                'uniq_columns': 'id',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': True,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 1
            },
            'log': {
                'platform': 'alfa',
                'report_name': 'log',
                'upload_table': 'log',
                'func_name': self.get_log,
                'uniq_columns': 'id',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': True,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 1
            },
            'task': {
                'platform': 'alfa',
                'report_name': 'task',
                'upload_table': 'task',
                'func_name': self.get_task,
                'uniq_columns': 'id',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': True,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 1
            },
            'lesson': {
                'platform': 'alfa',
                'report_name': 'lesson',
                'upload_table': 'lesson',
                'func_name': self.get_lesson,
                'uniq_columns': 'id',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': True,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 1
            },
            'update_customer': {
                'platform': 'alfa',
                'report_name': 'update_customer',
                'upload_table': 'customer',
                'func_name': self.update_customer,
                'uniq_columns': 'id',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': True,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 1
            },
            'update_group': {
                'platform': 'alfa',
                'report_name': 'update_group',
                'upload_table': 'group',
                'func_name': self.update_group,
                'uniq_columns': 'id',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': True,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 1
            },
            'update_pay': {
                'platform': 'alfa',
                'report_name': 'update_pay',
                'upload_table': 'pay',
                'func_name': self.update_pay,
                'uniq_columns': 'id',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': True,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 1
            },
            'update_lesson': {
                'platform': 'alfa',
                'report_name': 'update_lesson',
                'upload_table': 'lesson',
                'func_name': self.update_lesson,
                'uniq_columns': 'id',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': True,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 1
            },
            'all_customer': {
                'platform': 'alfa',
                'report_name': 'all_customer',
                'upload_table': 'customer',
                'func_name': self.all_customer,
                'uniq_columns': 'id',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': False,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 1
            },
            'all_regular_lesson': {
                'platform': 'alfa',
                'report_name': 'all_regular_lesson',
                'upload_table': 'regular_lesson',
                'func_name': self.all_regular_lesson,
                'uniq_columns': 'id',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': False,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 1
            },
            'all_group': {
                'platform': 'alfa',
                'report_name': 'all_group',
                'upload_table': 'group',
                'func_name': self.all_group,
                'uniq_columns': 'id',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': False,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 1
            },
            'all_pay': {
                'platform': 'alfa',
                'report_name': 'all_pay',
                'upload_table': 'pay',
                'func_name': self.all_pay,
                'uniq_columns': 'id',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': False,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 1
            },
            'all_lesson': {
                'platform': 'alfa',
                'report_name': 'all_lesson',
                'upload_table': 'lesson',
                'func_name': self.all_lesson,
                'uniq_columns': 'id',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': False,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 1
            },
        }

    def auth(self):
        try:
            url = f"{self.main_url.rstrip('/')}/v2api/auth/login"
            headers = {
                "Accept": "application/json",
                "Content-Type": "application/json",
                "X-APP-KEY": self.xappkey
            }
            data = {
                "email": self.email,
                "api_key": self.token
            }
            response = requests.post(url, headers=headers, data=json.dumps(data))
            code = response.status_code
            if code != 200:
                response.raise_for_status()
            else:
                result = response.json()
                self.access_token = result.get("token")
        except Exception as e:
            message = f'Платформа: ALFA. Имя: {self.add_name}. Функция: auth. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            raise


    def get_basic(self,url,filter_1=None, filter_2=None,filter_3=None, filter_4=None):
        try:
            all_data = []
            detector = True
            id0 = 0
            page = 0
            url = f"{self.main_url.rstrip('/')}/{url}"
            headers = {
                "X-ALFACRM-TOKEN": self.access_token,
                "Accept": "application/json",
                "Content-Type": "application/json",
            }
            while True:
                payload = {"page": page}
                if filter_1:
                    payload = payload | filter_1
                if filter_2 :
                    payload = payload | filter_2
                if filter_3:
                    payload = payload | filter_3
                if filter_4:
                    payload = payload | filter_4
                response = requests.post(url, headers=headers, data=json.dumps(payload))
                code = response.status_code
                if code == 401:
                    self.auth()
                    time.sleep(1)
                    response = requests.post(url, headers=headers, data=json.dumps(payload))
                    code = response.status_code
                if code != 200:
                    response.raise_for_status()
                else:
                    result = response.json()
                    data = result['items']
                    count = math.ceil(int(result['total'])/50)
                    print(f'URL: {url}. Всего страниц: {str(count)}. Страница: {str(page)}.' )
                    if len(data) > 0:
                        id1 = int(data[0]['id'])
                        if id0 == id1:
                            break
                        all_data += data
                if len(data) < 50:
                    break
                page +=1
                time.sleep(0.5)
                id0 = id1
            return self.common.replace_keys_in_data(all_data)
        except Exception as e:
            message = f'Платформа: ALFA. Имя: {self.add_name}. Функция: get_basic. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            raise

    def get_branch(self, date):
        try:
            final_result = self.get_basic('v2api/branch/index')
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_branch. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_branch. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    def get_subject(self, date):
        try:
            final_result = self.get_basic('v2api/subject/index')
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_subject. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_subject. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    def get_study_status(self, date):
        try:
            final_result = self.get_basic('v2api/study-status/index')
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_study_status. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_study_status. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    def get_lead_status(self, date):
        try:
            final_result = self.get_basic('v2api/lead-status/index')
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_lead_status. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_lead_status. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message
    def get_discount(self, date):
        try:
            final_result = self.get_basic('v2api/discount/index')
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_discount. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_discount. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message
    def get_lead_source(self, date):
        try:
            final_result = self.get_basic('v2api/lead-source/index')
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_lead_source. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_lead_source. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    def get_location(self, date):
        try:
            final_result = []
            for branch in self.branches_list:
                final_result += self.get_basic(f'v2api/{str(branch).strip()}/location/index')
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_location. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_location. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    def get_group(self, date):
        try:
            final_result = []
            for branch in self.branches_list:
                final_result += self.get_basic(f'v2api/{str(branch).strip()}/group/index',{'created_at_to': self.common.flip_date(date)},{'created_at_from': self.common.flip_date(date)},{'removed': 1})
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_group. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_group. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    def get_customer(self, date):
        try:
            final_result = []
            for branch in self.branches_list:
                final_result += self.get_basic(f'v2api/{str(branch).strip()}/customer/index', {'date_to': date},{'date_from': date}, {'removed': 1},{'is_study': 2})
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_customer. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_customer. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message
    def get_regular_lesson(self, date):
        try:
            final_result = []
            for branch in self.branches_list:
                final_result += self.get_basic(f'v2api/{str(branch).strip()}/regular-lesson/index', {'date_to': date},{'date_from': date})
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_regular_lesson. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_regular_lesson. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message
    def get_tariff(self, date):
        try:
            final_result = []
            for branch in self.branches_list:
                final_result += self.get_basic(f'v2api/{str(branch).strip()}/tariff/index')
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_tariff. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_tariff. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message
    def get_task(self, date):
        try:
            final_result = []
            for branch in self.branches_list:
                final_result += self.get_basic(f'v2api/{str(branch).strip()}/task/index', {'due_date_from': date},{'due_date_to': date})
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_task. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_task. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    def get_task_null(self, date):
        try:
            final_result = []
            for branch in self.branches_list:
                final_result += self.get_basic(f'v2api/{str(branch).strip()}/task/index', {'due_date_is_null': True})
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_task_null. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_task_null. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    def get_lesson(self, date):
        try:
            final_result = []
            for branch in self.branches_list:
                final_result += self.get_basic(f'v2api/{str(branch).strip()}/lesson/index', {'date_to': date},{'date_from': date})
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_lesson. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_lesson. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    def get_pay(self, date):
        try:
            final_result = []
            for branch in self.branches_list:
                final_result += self.get_basic(f'v2api/{str(branch).strip()}/pay/index', {'date_to': self.common.shift_date(date,-1)},{'date_from': date})
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_pay. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_pay. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    def get_log(self, date):
        try:
            final_result = []
            for branch in self.branches_list:
                final_result += self.get_basic(f'v2api/{str(branch).strip()}/log/index', {'date_to': self.common.flip_date(date)},{'date_from': self.common.flip_date(date)})
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_log. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_log. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    def update_customer(self, date):
        try:
            final_result = []
            for branch in self.branches_list:
                final_result += self.get_basic(f'v2api/{str(branch).strip()}/customer/index', {'updated_at_to': self.common.flip_date(date)},{'updated_at_from': self.common.flip_date(date)},{'removed': 1},{'is_study': 2})
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: update_customer. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: update_customer. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    def update_group(self, date):
        try:
            final_result = []
            for branch in self.branches_list:
                final_result += self.get_basic(f'v2api/{str(branch).strip()}/group/index', {'updated_at_to': self.common.flip_date(date)},{'updated_at_from': self.common.flip_date(date)},{'removed': 1})
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: update_group. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: update_group. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    def update_lesson(self, date):
        try:
            final_result = []
            for branch in self.branches_list:
                final_result += self.get_basic(f'v2api/{str(branch).strip()}/lesson/index', {'updated_at_to': self.common.flip_date(date)},{'updated_at_from': self.common.flip_date(date)})
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: update_lesson. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: update_lesson. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    def update_pay(self, date):
        try:
            final_result = []
            for branch in self.branches_list:
                final_result += self.get_basic(f'v2api/{str(branch).strip()}/pay/index', {'updated_at_to': self.common.flip_date(date)},{'updated_at_from': self.common.flip_date(date)})
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: update_pay. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: update_pay. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    def all_customer(self, date):
        try:
            final_result = []
            for branch in self.branches_list:
                final_result += self.get_basic(f'v2api/{str(branch).strip()}/customer/index',{'removed': 1},{'is_study': 2})
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: all_customer. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: all_customer. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message
    def all_regular_lesson(self, date):
        try:
            final_result = []
            for branch in self.branches_list:
                final_result += self.get_basic(f'v2api/{str(branch).strip()}/regular-lesson/index')
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: all_regular_lesson. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: all_regular_lesson. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    def all_group(self, date):
        try:
            final_result = []
            for branch in self.branches_list:
                final_result += self.get_basic(f'v2api/{str(branch).strip()}/group/index',{'removed': 1})
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: all_group. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: all_group. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    def all_lesson(self, date):
        try:
            final_result = []
            for branch in self.branches_list:
                final_result += self.get_basic(f'v2api/{str(branch).strip()}/lesson/index')
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: all_lesson. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: all_lesson. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    def all_pay(self, date):
        try:
            final_result = []
            for branch in self.branches_list:
                final_result += self.get_basic(f'v2api/{str(branch).strip()}/pay/index')
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: all_pay. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: all_pay. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    def get_teacher(self, date):
        try:
            final_result = []
            for branch in self.branches_list:
                final_result += self.get_basic(f'v2api/{str(branch).strip()}/teacher/index', {'removed':1})
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_teacher. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: ALFA. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_teacher. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message


    def collecting_manager(self):
        self.auth()
        report_list = self.reports.replace(' ', '').lower().split(',')
        for report in report_list:
            self.clickhouse = Clickhouse(self.bot_token, self.chat_list, self.message_type, self.host, self.port,
                                         self.username, self.password,
                                         self.database, self.start, self.add_name, self.err429, self.backfill_days,
                                         self.platform)
            self.clickhouse.collecting_report(
                self.source_dict[report]['platform'],
                self.source_dict[report]['report_name'],
                self.source_dict[report]['upload_table'],
                self.source_dict[report]['func_name'],
                self.source_dict[report]['uniq_columns'],
                self.source_dict[report]['partitions'],
                self.source_dict[report]['merge_type'],
                self.source_dict[report]['refresh_type'],
                self.source_dict[report]['history'],
                self.source_dict[report]['frequency'],
                self.source_dict[report]['delay']
            )





from .common import Common
from .clickhouse import Clickhouse
from .wb_reklama import WBreklama
import requests
from datetime import datetime,timedelta
import clickhouse_connect
import pandas as pd
import os
from dateutil import parser
import time
import hashlib
from io import StringIO
import json


class WBbyDate:
    def __init__(self, bot_token:str, chats:str, message_type: str, subd: str,
                 host: str, port: str, username: str, password: str, database: str,
                 add_name: str, token: str ,  start: str, backfill_days: int, reports :str):
        self.bot_token = bot_token
        self.chat_list = chats.replace(' ', '').split(',')
        self.message_type = message_type
        self.common = Common(self.bot_token, self.chat_list, self.message_type)
        self.token = token
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.database = database
        self.subd = subd
        self.add_name = self.common.transliterate_key(add_name)
        self.now = datetime.now()
        self.today = datetime.now().date()
        self.start = start
        self.reports = reports
        self.backfill_days = backfill_days
        self.platform = 'wb'

        self.err429 = False
        self.source_dict = {
            'realized': {
                'platform': 'wb',
                'report_name': 'realized',
                'upload_table': 'realized',
                'func_name': self.get_realized,
                'uniq_columns': 'realizationreport_id,rrd_id',
                'partitions': 'realizationreport_id',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': True,
                'frequency': 'Monday',  # '2dayOfMonth,Friday'
                'delay': 60
            },
            'orders': {
                'platform': 'wb',
                'report_name': 'orders',
                'upload_table': 'orders',
                'func_name': self.get_orders,
                'uniq_columns': 'date,srid',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': True,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 60
            },
            'incomes': {
                'platform': 'wb',
                'report_name': 'incomes',
                'upload_table': 'incomes',
                'func_name': self.get_incomes,
                'uniq_columns': 'incomeId,barcode',
                'partitions': '',
                'merge_type': 'MergeTree',
                'refresh_type': 'delete_all',
                'history': False,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 60
            },
            'sales': {
                'platform': 'wb',
                'report_name': 'sales',
                'upload_table': 'sales',
                'func_name': self.get_sales,
                'uniq_columns': 'date,saleID',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': True,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 60
            },
            'orders_changes': {
                'platform': 'wb',
                'report_name': 'orders_changes',
                'upload_table': 'orders',
                'func_name': self.get_orders_changes,
                'uniq_columns': 'date,srid',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': False,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 60
            },
            'sales_changes': {
                'platform': 'wb',
                'report_name': 'sales_changes',
                'upload_table': 'sales',
                'func_name': self.get_sales_changes,
                'uniq_columns': 'date,saleID',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': False,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 60
            },
            'stocks': {
                'platform': 'wb',
                'report_name': 'stocks',
                'upload_table': 'stocks',
                'func_name': self.get_stocks,
                'uniq_columns': 'lastChangeDate',
                'partitions': '',
                'merge_type': 'MergeTree',
                'refresh_type': 'delete_all',
                'history': False,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 60
            },
            'stocks_history': {
                'platform': 'wb',
                'report_name': 'stocks_history',
                'upload_table': 'stocks_history',
                'func_name': self.get_stocks,
                'uniq_columns': 'lastChangeDate',
                'partitions': '',
                'merge_type': 'MergeTree',
                'refresh_type': 'nothing',
                'history': False,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 60
            },
            'paid_storage': {
                'platform': 'wb',
                'report_name': 'paid_storage',
                'upload_table': 'paid_storage',
                'func_name': self.get_paid_storage,
                'uniq_columns': 'date',
                'partitions': 'date',
                'merge_type': 'MergeTree',
                'refresh_type': 'delete_date',
                'history': True,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 60
            },
            'nmreport': {
                'platform': 'wb',
                'report_name': 'nmreport',
                'upload_table': 'nmreport',
                'func_name': self.get_nmreport,
                'uniq_columns': 'nmID,statistics_selectedPeriod_begin',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': True,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 60
            },
        }

    def create_ps_report(self, api_key, date1, date2):
        try:
            url = "https://seller-analytics-api.wildberries.ru/api/v1/paid_storage"
            headers = {"Authorization": api_key}
            params = {"dateFrom": date1, "dateTo": date2, }
            response = requests.get(url, headers=headers, params=params)
            code = response.status_code
            if code == 200:
                return response.json()['data']['taskId']
            else:
                response.raise_for_status()
        except Exception as e:
                message = f'Платформа: WB. Имя: {self.add_name}. Даты: {date1}-{date2}. Функция: create_ps_report. Ошибка: {e}.'
                self.common.log_func(self.bot_token, self.chat_list, message, 3)
                return message


    def ps_report_status(self, api_key, task_id):
        try:
            url = f"https://seller-analytics-api.wildberries.ru/api/v1/paid_storage/tasks/{task_id}/status"
            headers = {"Authorization": api_key}
            response = requests.get(url, headers=headers)
            code = response.status_code
            if code == 200:
                return response.json()['data']['status']
            else:
                response.raise_for_status()
        except Exception as e:
                message = f'Платформа: WB. Имя: {self.add_name}. Функция: ps_report_status. Ошибка: {e}.'
                self.common.log_func(self.bot_token, self.chat_list, message, 3)
                return message


    def get_ps_report(self, api_key, task_id):
        try:
            url = f"https://seller-analytics-api.wildberries.ru/api/v1/paid_storage/tasks/{task_id}/download"
            headers = {"Authorization": api_key}
            response = requests.get(url, headers=headers)
            code = response.status_code
            if code == 200:
                return response.json()
            else:
                response.raise_for_status()
        except Exception as e:
                message = f'Платформа: WB. Имя: {self.add_name}. Функция: get_ps_report. Ошибка: {e}.'
                self.common.log_func(self.bot_token, self.chat_list, message, 3)
                return message


    def get_paid_storage(self, date):
        try:
            task = self.create_ps_report(self.token, date, date)
            for t in range(20):
                time.sleep(10)
                if self.ps_report_status(self.token, task) =='done':
                    message = f'Платформа: WB. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_paid_storage. Результат: ОК'
                    self.common.log_func(self.bot_token, self.chat_list, message, 1)
                    return self.get_ps_report(self.token, task)
        except Exception as e:
            message = f'Платформа: WB. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_paid_storage. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message


    # дата+токен -> список словарей с заказами (данные)
    def get_orders(self, date):
        try:
            date_rfc3339 = f"{date}T00:00:00.000Z"
            url = "https://statistics-api.wildberries.ru/api/v1/supplier/orders"
            headers = {
                "Authorization": self.token,
            }
            params = {
                "dateFrom": date_rfc3339,
                "flag": 1,  # Для получения всех заказов на указанную дату
            }
            response = requests.get(url, headers=headers, params=params)
            code = response.status_code
            print(code)
            if code == 429:
                self.err429 = True
            if code == 200:
                final_result = response.json()

            else:
                response.raise_for_status()
            message = f'Платформа: WB. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_orders. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: WB. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_orders. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    def get_incomes(self, date):
        try:
            date_rfc3339 = f"{self.start}T00:00:00.000Z"
            url = "https://statistics-api.wildberries.ru/api/v1/supplier/incomes"
            headers = {
                "Authorization": self.token,
            }
            params = {
                "dateFrom": date_rfc3339
            }
            response = requests.get(url, headers=headers, params=params, timeout=200)
            code = response.status_code
            if code == 429:
                self.err429 = True
            if code == 200:
                json_data = response.json()
                if not json_data or all(not item for item in json_data if isinstance(json_data, list)):
                    raise ValueError("Получен пустой Json")
                final_result = json_data
            else:
                response.raise_for_status()
            message = f'Платформа: WB. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_incomes. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: WB. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_incomes. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    def get_orders_changes(self, date):
        try:
            date_rfc3339 = f"{date}T00:00:00.000Z"
            url = "https://statistics-api.wildberries.ru/api/v1/supplier/orders"
            headers = {"Authorization": self.token}
            params = {"dateFrom": date_rfc3339}
            response = requests.get(url, headers=headers, params=params)
            code = response.status_code
            if code == 429:
                self.err429 = True
            if code == 200:
                final_result = response.json()
            else:
                response.raise_for_status()
            message = f'Платформа: WB. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_orders_changes. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: WB. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_orders_changes. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    # дата+токен -> список словарей с заказами (данные)
    def get_sales(self, date):
        try:
            url = 'https://statistics-api.wildberries.ru/api/v1/supplier/sales'
            headers = {
                'Authorization': f'Bearer {self.token}'
            }
            params = {
                'dateFrom': date,
                "flag": 1,
            }
            response = requests.get(url, headers=headers, params=params)
            code = response.status_code
            if code == 429:
                self.err429 = True
            if code == 200:
                final_result = response.json()
            else:
                response.raise_for_status()
            message = f'Платформа: WB. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_sales. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: WB. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_sales. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    def get_sales_changes(self, date):
        try:
            url = 'https://statistics-api.wildberries.ru/api/v1/supplier/sales'
            headers = {'Authorization': f'Bearer {self.token}'}
            params = {'dateFrom': date}
            response = requests.get(url, headers=headers, params=params)
            code = response.status_code
            if code == 429:
                self.err429 = True
            if code == 200:
                final_result = response.json()
            else:
                response.raise_for_status()
            message = f'Платформа: WB. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_sales_changes. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: WB. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_sales_changes. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    # дата+токен -> список словарей с заказами (данные)
    def get_realized(self, date):
        try:
            url = 'https://statistics-api.wildberries.ru/api/v5/supplier/reportDetailByPeriod'
            headers = {'Authorization': f'Bearer {self.token}'}
            params = {'dateFrom': self.common.shift_date(date,7), 'dateTo': self.common.shift_date(date,1)}
            response = requests.get(url, headers=headers, params=params)
            code = response.status_code
            if code == 429:
                self.err429 = True
            if code == 200:
                final_result = response.json()
            else:
                response.raise_for_status()
            message = f'Платформа: WB. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_realized. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: WB. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_realized. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    def get_stocks(self, date):
        try:
            # Преобразуем дату в формат RFC3339
            date_rfc3339 = f"{self.start}T00:00:00.000Z"
            url = "https://statistics-api.wildberries.ru/api/v1/supplier/stocks"
            headers = {
                "Authorization": self.token,
            }
            params = {
                "dateFrom": date_rfc3339,
            }
            response = requests.get(url, headers=headers, params=params)
            code = response.status_code
            if code == 429:
                self.err429 = True
            if code == 200:
                final_result = response.json()
            else:
                response.raise_for_status()
            message = f'Платформа: WB. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_stocks. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return     final_result
        except Exception as e:
            message = f'Платформа: WB. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_stocks. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    def get_nmreport(self, date):
        try:
            url = "https://seller-analytics-api.wildberries.ru/api/v2/nm-report/detail"
            headers = {
                "Authorization": self.token,
                "Content-Type": "application/json"
            }
            page = 1
            all_cards = []  # Хранилище для всех карточек товара
            begin_date = f"{date} 00:00:00"
            end_date = f"{date} 23:59:59"
            while True:
                payload = {
                    "period": {
                        "begin": begin_date,
                        "end": end_date
                    },
                    "page": page
                }
                response = requests.post(url, headers=headers, json=payload)
                code = response.status_code
                if code == 200:
                    data = response.json().get('data', {})
                    cards = data.get('cards', [])
                    all_cards.extend(cards)  # Добавляем карточки на текущей странице
                    is_next_page = data.get('isNextPage', False)
                    if not is_next_page:
                        break  # Если страниц больше нет, выходим из цикла
                    page += 1  # Переходим на следующую страницу
                else:
                    response.raise_for_status()
            message = f'Платформа: WB. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_nmreport. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return self.common.spread_table(self.common.spread_table(self.common.spread_table(all_cards)))
        except Exception as e:
            message = f'Платформа: WB. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_nmreport. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    # тип отчёта, дата -> данные в CH
    def collecting_manager(self):
        report_list = self.reports.replace(' ', '').lower().split(',')
        for report in report_list:
            if report == 'reklama':
                self.reklama = WBreklama(self.bot_token, self.chat_list, self.message_type, self.subd, self.add_name, self.token, self.host, self.port, self.username, self.password,
                                             self.database, self.start,  self.backfill_days,)
                self.reklama.wb_reklama_collector()
            else:
                self.clickhouse = Clickhouse(self.bot_token, self.chat_list, self.message_type, self.host, self.port, self.username, self.password,
                                             self.database, self.start, self.add_name, self.err429, self.backfill_days, self.platform)
                self.clickhouse.collecting_report(
                    self.source_dict[report]['platform'],
                    self.source_dict[report]['report_name'],
                    self.source_dict[report]['upload_table'],
                    self.source_dict[report]['func_name'],
                    self.source_dict[report]['uniq_columns'],
                    self.source_dict[report]['partitions'],
                    self.source_dict[report]['merge_type'],
                    self.source_dict[report]['refresh_type'],
                    self.source_dict[report]['history'],
                    self.source_dict[report]['frequency'],
                    self.source_dict[report]['delay']
                )












from .common import Common
from .clickhouse import Clickhouse
from .ozon_reklama import OZONreklama
import requests
from datetime import datetime,timedelta
import clickhouse_connect
import pandas as pd
import os
from dateutil import parser
import time
import hashlib
from io import StringIO
import json
from dateutil.relativedelta import relativedelta


class OZONbyDate:
    def __init__(self,  bot_token:str, chats:str, message_type: str, subd: str,
                 host: str, port: str, username: str, password: str, database: str,
                                  add_name: str, clientid:str, token: str ,  start: str, backfill_days: int, reports :str):
        self.bot_token = bot_token
        self.chat_list = chats.replace(' ', '').split(',')
        self.message_type = message_type
        self.common = Common(self.bot_token, self.chat_list, self.message_type)
        self.clientid = clientid
        self.token = token
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.database = database
        self.subd = subd
        self.add_name = self.common.transliterate_key(add_name)
        self.now = datetime.now()
        self.today = datetime.now().date()
        self.start = start
        self.reports = reports
        self.backfill_days = backfill_days
        self.platform = 'ozon'
        self.err429 = False
        self.source_dict = {
            'transactions': {
                'platform': 'ozon',
                'report_name': 'transactions',
                'upload_table': 'transactions',
                'func_name': self.get_transactions,
                'uniq_columns': 'operation_date, operation_id',
                'partitions': 'operation_date',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': True,
                'frequency': 'daily', # '2dayOfMonth,Friday'
                'delay': 30
            },
            'stocks': {
                'platform': 'ozon',
                'report_name': 'stocks',
                'upload_table': 'stocks',
                'func_name': self.get_stock_on_warehouses,
                'uniq_columns': 'sku',
                'partitions': 'warehouse_name',
                'merge_type': 'MergeTree',
                'refresh_type': 'delete_all',
                'history': False,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 30
            },
            'stocks_history': {
                'platform': 'ozon',
                'report_name': 'stocks_history',
                'upload_table': 'stocks_history',
                'func_name': self.get_stock_on_warehouses,
                'uniq_columns': 'sku',
                'partitions': 'warehouse_name',
                'merge_type': 'MergeTree',
                'refresh_type': 'nothing',
                'history': False,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 30
            },
            'products': {
                'platform': 'ozon',
                'report_name': 'products',
                'upload_table': 'products',
                'func_name': self.get_all_products,
                'uniq_columns': 'product_id',
                'partitions': '',
                'merge_type': 'MergeTree',
                'refresh_type': 'delete_all',
                'history': False,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 30
            },
            'returns_fbo': {
                'platform': 'ozon',
                'report_name': 'returns_fbo',
                'upload_table': 'returns_fbo',
                'func_name': self.get_all_returns_fbo,
                'uniq_columns': 'return_id',
                'partitions': '',
                'merge_type': 'MergeTree',
                'refresh_type': 'delete_all',
                'history': False,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 30
            },
            'returns_fbs': {
                'platform': 'ozon',
                'report_name': 'returns_fbs',
                'upload_table': 'returns_fbs',
                'func_name': self.get_all_returns_fbs,
                'uniq_columns': 'id',
                'partitions': '',
                'merge_type': 'MergeTree',
                'refresh_type': 'delete_all',
                'history': False,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 30
            },
            'realization': {
                'platform': 'ozon',
                'report_name': 'realization',
                'upload_table': 'realization',
                'func_name': self.get_realization,
                'uniq_columns': 'year_month,rowNumber',
                'partitions': 'year_month',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': True,
                'frequency': '6',  # '2,Friday'
                'delay': 30
            },
            'postings_fbo': {
                'platform': 'ozon',
                'report_name': 'postings_fbo',
                'upload_table': 'postings_fbo',
                'func_name': self.get_postings_fbo,
                'uniq_columns': 'posting_number,created_at',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': True,
                'frequency': 'daily',  # '2,Friday'
                'delay': 30
            },
            'finance_details': {
                'platform': 'ozon',
                'report_name': 'finance_details',
                'upload_table': 'finance_details',
                'func_name': self.get_finance_details,
                'uniq_columns': 'period_id',
                'partitions': 'period_id',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': True,
                'frequency': '1,16',  # '2,Friday'
                'delay': 30
            },
            'finance_cashflow': {
                'platform': 'ozon',
                'report_name': 'finance_cashflow',
                'upload_table': 'finance_cashflow',
                'func_name': self.get_finance_cashflow,
                'uniq_columns': 'period_id',
                'partitions': 'period_id',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': True,
                'frequency': '1,16',  # '2,Friday'
                'delay': 30
            },
        }


    def get_transaction_page_count(self, date):
        try:
            url = "https://api-seller.ozon.ru/v3/finance/transaction/list"
            headers = {"Client-Id": self.clientid,
                       "Api-Key": self.token,
                       "Content-Type": "application/json"}
            payload = {
                "filter": {
                    "date": {"from": f"{date}T00:00:00.000Z",
                             "to": f"{date}T23:59:59.999Z"},
                    "operation_type": [],
                    "posting_number": "",
                    "transaction_type": "all"},
                "page": 1,
                "page_size": 1000
            }
            response = requests.post(url, headers=headers, data=json.dumps(payload))
            code = response.status_code
            if code == 429:
                self.err429 = True
            if code == 200:
                page_count = response.json()['result']['page_count']
                return page_count
            else:
                response.raise_for_status()
        except Exception as e:
            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_transaction_page_count. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    def get_transactions(self, date):
        try:
            url = "https://api-seller.ozon.ru/v3/finance/transaction/list"
            headers = {"Client-Id": self.clientid,
                       "Api-Key": self.token,
                       "Content-Type": "application/json"}
            page_count = int(self.get_transaction_page_count(date))
            operations = []
            for page in range(1, page_count + 1):
                payload = {
                    "filter": {
                        "date": {"from": f"{date}T00:00:00.000Z",
                                 "to": f"{date}T23:59:59.999Z"},
                        "operation_type": [],
                        "posting_number": "",
                        "transaction_type": "all"},
                    "page": page,
                    "page_size": 1000
                }
                response = requests.post(url, headers=headers, data=json.dumps(payload))
                code = response.status_code
                if code == 429:
                    self.err429 = True
                if code == 200:
                    operations += response.json()['result']['operations']
                else:
                    response.raise_for_status()
                time.sleep(2)
            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_transactions. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return operations
        except Exception as e:
            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_transactions. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    def get_stock_on_warehouses(self, date):
        try:
            url = "https://api-seller.ozon.ru/v2/analytics/stock_on_warehouses"
            headers = {
                "Client-Id": self.clientid,
                "Api-Key": self.token,
                "Content-Type": "application/json"
            }
            offset = 0
            limit = 1000
            all_rows = []
            while True:
                payload = {
                    "limit": limit,
                    "offset": offset,
                    "warehouse_type": "ALL"
                }
                response = requests.post(url, headers=headers, data=json.dumps(payload))
                code = response.status_code
                if code == 429:
                    self.err429 = True
                if code == 200:
                    rows = response.json().get('result', {}).get('rows', [])
                    if not rows:
                        # Прекращаем цикл, если ответ пустой
                        break
                    all_rows.extend(rows)  # Добавляем все элементы 'rows' в общий список
                    offset += limit
                else:
                    response.raise_for_status()
            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_stock_on_warehouses. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return all_rows
        except Exception as e:
            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_stock_on_warehouses. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    def get_all_products(self, date):
        try:
            url = "https://api-seller.ozon.ru/v2/product/list"
            headers = {
                "Client-Id": self.clientid,
                "Api-Key": self.token,
                "Content-Type": "application/json"
            }
            limit = 1000
            last_id = ""  # Инициализируем last_id пустым значением для первого запроса
            all_items = []  # Список для хранения всех продуктов
            while True:
                payload = {
                    "last_id": last_id,
                    "limit": limit
                }
                response = requests.post(url, headers=headers, data=json.dumps(payload))
                code = response.status_code
                if code == 429:
                    self.err429 = True
                if code == 200:
                    result = response.json().get('result', {})
                    items = result.get('items', [])
                    if not items:
                        break
                    all_items.extend(items)
                    if len(items) < limit:
                        break
                    last_id = result.get('last_id', "")
                else:
                    response.raise_for_status()
            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_all_products. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return all_items
        except Exception as e:
            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_all_products. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    def get_all_returns_fbo(self, date):
        try:
            url = "https://api-seller.ozon.ru/v3/returns/company/fbo"
            headers = {
                "Client-Id": self.clientid,
                "Api-Key": self.token,
                "Content-Type": "application/json"
            }
            limit = 1000  # Можно задать желаемый лимит записей на один запрос
            last_id = 0  # Инициализируем last_id с начальным значением 0
            all_returns = []  # Список для хранения всех возвратов
            while True:
                payload = {
                    "last_id": last_id,
                    "limit": limit
                }
                response = requests.post(url, headers=headers, data=json.dumps(payload))
                code = response.status_code
                if code == 429:
                    self.err429 = True
                if code == 200:
                    result = response.json()
                    returns = result.get('returns', [])
                    if not returns:
                        break
                    all_returns.extend(returns)  # Добавляем все элементы 'returns' в общий список
                    if len(returns) < limit:
                        break
                    last_id = result.get('last_id', 0)
                else:
                    response.raise_for_status()
            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_all_returns_fbo. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return all_returns  # Возвращаем итоговый список из 'returns'
        except Exception as e:
            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_all_returns_fbo. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    def get_all_returns_fbs(self, date):
        try:
            url = "https://api-seller.ozon.ru/v3/returns/company/fbs"
            headers = {
                "Client-Id": self.clientid,
                "Api-Key": self.token,
                "Content-Type": "application/json"
            }
            limit = 1000  # Устанавливаем лимит на количество записей за запрос
            last_id = 0  # Инициализируем last_id с начальным значением 0
            all_returns = []  # Список для хранения всех возвратов
            while True:
                payload = {
                    "limit": limit,
                    "last_id": last_id
                }
                response = requests.post(url, headers=headers, data=json.dumps(payload))
                code = response.status_code
                if code == 429:
                    self.err429 = True
                if code == 200:
                    result = response.json()
                    returns = result.get('returns', [])
                    if not returns:
                        break
                    all_returns.extend(returns)
                    if len(returns) < limit:
                        break
                    last_id = result.get('last_id', 0)
                else:
                    response.raise_for_status()
            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_all_returns_fbs. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return all_returns
        except Exception as e:
            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_all_returns_fbs. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    def get_realization(self, date):
        try:
            real_date = datetime.strptime(date, "%Y-%m-%d")
            last_month_date = real_date - relativedelta(months=1)
            previous_month = last_month_date.month
            previous_year = last_month_date.year
            yyyy_mm = f"{previous_year}-{str(previous_month).zfill(2)}-01"
            final_data = []
            data = {
                "month": previous_month,
                "year": previous_year
            }
            headers = {
                "Client-Id": self.clientid,
                "Api-Key": self.token,
                "Content-Type": "application/json"
            }
            url = "https://api-seller.ozon.ru/v2/finance/realization"
            response = requests.post(url, json=data, headers=headers)
            code = response.status_code
            if code == 429:
                self.err429 = True
            if code == 200:
                result = response.json().get('result', {}).get('rows', [])
                for row in result:
                    row['year_month'] = yyyy_mm
                    final_data.append(row)
                final_result = self.common.spread_table(final_data)
            else:
                response.raise_for_status()
            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_realization. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_realization. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message


    def get_postings_fbo(self, date):
        try:
            url = "https://api-seller.ozon.ru/v2/posting/fbo/list"
            headers = {
                "Client-Id": self.clientid,
                "Api-Key": self.token,
                "Content-Type": "application/json"
            }
            offset = 0
            limit = 1000
            all_postings = []  # Список для хранения всех отправлений
            while True:
                payload = {
                    "dir": "ASC",
                    "filter": {
                        "since": f"{date}T00:00:00.000Z",  # Дата с началом дня
                        "status": "",
                        "to": f"{date}T23:59:59.999Z"  # Дата с концом дня
                    },
                    "limit": limit,
                    "offset": offset,
                    "with": {
                        "analytics_data": True,
                        "financial_data": True
                    }
                }
                response = requests.post(url, headers=headers, data=json.dumps(payload))
                code = response.status_code
                if code == 429:
                    self.err429 = True
                if code == 200:
                    result = response.json().get('result', [])
                    if not result:
                        break
                    all_postings.extend(result)
                    offset += limit
                else:
                    response.raise_for_status()
            all_postings_with_date = []
            for dict in all_postings:
                dict['date']=date
                all_postings_with_date.append(dict)
            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_postings_fbo. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return self.common.spread_table(all_postings_with_date)
        except Exception as e:
            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_postings_fbo. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    def get_date_range(self, date):
        try:
            date_obj = datetime.strptime(date, "%Y-%m-%d")
            day = date_obj.day
            if day == 1:
                # С 16-го числа предыдущего месяца по конец предыдущего месяца
                last_day_of_prev_month = date_obj.replace(day=1) - timedelta(days=1)
                start_date = last_day_of_prev_month.replace(day=16)
                end_date = last_day_of_prev_month
            else:
                # С 1-го по 15-е число текущего месяца
                start_date = date_obj.replace(day=1)
                end_date = date_obj.replace(day=15)
            return start_date.strftime("%Y-%m-%dT00:00:00.000Z"), end_date.strftime("%Y-%m-%dT23:59:59.999Z")
        except Exception as e:
            message = f'Платформа: OZON. Имя: {self.add_name}. Даты: {str(date)}. Функция: get_date_range. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message


    def get_finance_total_pages(self, start_date, end_date):
        try:
            url = "https://api-seller.ozon.ru/v1/finance/cash-flow-statement/list"
            headers = {
                "Client-Id": self.clientid,
                "Api-Key": self.token,
                "Content-Type": "application/json"
            }
            payload = {
                "date": {
                    "from": start_date,
                    "to": end_date
                },
                "with_details": True,
                "page": 1,
                "page_size": 1
            }
            response = requests.post(url, headers=headers, data=json.dumps(payload))
            code = response.status_code
            if code == 429:
                self.err429 = True
            if code == 200:
                return response.json().get('page_count', 1)
            else:
                response.raise_for_status()
        except Exception as e:
            message = f'Платформа: OZON. Имя: {self.add_name}. Даты: {str(start_date)}-{str(end_date)}. Функция: get_finance_total_pages. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message


    def get_finance_details(self, date):
        try:
            start_date, end_date = self.get_date_range(date)
            total_pages = self.get_finance_total_pages(start_date, end_date)
            url = "https://api-seller.ozon.ru/v1/finance/cash-flow-statement/list"
            headers = {
                "Client-Id": self.clientid,
                "Api-Key": self.token,
                "Content-Type": "application/json"
            }
            all = []  # Список для хранения всех cash_flows
            for page in range(1, total_pages + 1):
                payload = {
                    "date": {
                        "from": start_date,
                        "to": end_date
                    },
                    "with_details": True,
                    "page": page,
                    "page_size": 1000
                }
                response = requests.post(url, headers=headers, data=json.dumps(payload))
                code = response.status_code
                if code == 429:
                    self.err429 = True
                if code == 200:
                    result = response.json().get('result', {}).get('details', [])
                    all.extend(result)
                else:
                    response.raise_for_status()
            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_finance_details. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return self.common.spread_table(all)
        except Exception as e:
            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_finance_details. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message

    def get_finance_cashflow(self, date):
        try:
            start_date, end_date = self.get_date_range(date)
            total_pages = self.get_finance_total_pages(start_date, end_date)
            url = "https://api-seller.ozon.ru/v1/finance/cash-flow-statement/list"
            headers = {
                "Client-Id": self.clientid,
                "Api-Key": self.token,
                "Content-Type": "application/json"
            }
            all = []  # Список для хранения всех cash_flows
            for page in range(1, total_pages + 1):
                payload = {
                    "date": {
                        "from": start_date,
                        "to": end_date
                    },
                    "with_details": True,
                    "page": page,
                    "page_size": 1000
                }
                response = requests.post(url, headers=headers, data=json.dumps(payload))
                code = response.status_code
                if code == 429:
                    self.err429 = True
                if code == 200:
                    result = response.json().get('result', {}).get('cash_flows', [])
                    all.extend(result)
                else:
                    response.raise_for_status()
            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_finance_details. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return self.common.spread_table(all)
        except Exception as e:
            message = f'Платформа: OZON. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_finance_cashflow. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message


    def collecting_manager(self):
        report_list = self.reports.replace(' ', '').lower().split(',')
        for report in report_list:
            if report == 'reklama':
                self.reklama = OZONreklama(self.bot_token, self.chat_list, self.message_type, self.subd, self.add_name, self.clientid, self.token,
                                           self.host, self.port, self.username, self.password,                                              self.database, self.start,  self.backfill_days)
                self.reklama.ozon_reklama_collector()
            else:
                self.clickhouse = Clickhouse(self.bot_token, self.chat_list, self.message_type, self.host, self.port, self.username, self.password,
                                             self.database, self.start, self.add_name, self.err429, self.backfill_days, self.platform)
                self.clickhouse.collecting_report(
                    self.source_dict[report]['platform'],
                    self.source_dict[report]['report_name'],
                    self.source_dict[report]['upload_table'],
                    self.source_dict[report]['func_name'],
                    self.source_dict[report]['uniq_columns'],
                    self.source_dict[report]['partitions'],
                    self.source_dict[report]['merge_type'],
                    self.source_dict[report]['refresh_type'],
                    self.source_dict[report]['history'],
                    self.source_dict[report]['frequency'],
                    self.source_dict[report]['delay']
                )







from .common import Common
from .clickhouse import Clickhouse
from .ozon_reklama import OZONreklama
import requests
from datetime import datetime,timedelta
import clickhouse_connect
import pandas as pd
from transliterate import translit
import os
from dateutil import parser
import time
import hashlib
from io import StringIO
import json
from dateutil.relativedelta import relativedelta


class GCbyDate:
    def __init__(self, bot_token:str, chats:str, message_type: str, subd: str,
                 host: str, port: str, username: str, password: str, database: str,
                 add_name: str, clientid:str, token: str , start: str, group_id: str, reports :str):
        self.bot_token = bot_token
        self.chat_list = chats.replace(' ','').split(',')
        self.message_type  = message_type
        self.common = Common(self.bot_token, self.chat_list, self.message_type)
        self.clientid = clientid
        self.token = token
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.database = database
        self.subd = subd
        self.add_name = self.common.transliterate_key(add_name)
        self.now = datetime.now()
        self.today = datetime.now().date()
        self.start = start
        self.reports = reports
        self.group_id = group_id
        self.backfill_days = 3
        self.platform = 'gc'
        self.err429 = False
        self.source_dict = {
            'users': {
                'platform': 'gc',
                'report_name': 'users',
                'upload_table': 'users',
                'func_name': self.get_users,
                'uniq_columns': 'id',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': False,
                'frequency': 'daily', # '2dayOfMonth,Friday'
                'delay': 30
            },
            'deals': {
                'platform': 'gc',
                'report_name': 'deals',
                'upload_table': 'deals',
                'func_name': self.get_deals,
                'uniq_columns': 'id_zakaza',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': False,
                'frequency': 'daily', # '2dayOfMonth,Friday'
                'delay': 30
            },
            'payments': {
                'platform': 'gc',
                'report_name': 'payments',
                'upload_table': 'payments',
                'func_name': self.get_payments,
                'uniq_columns': 'nomer',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': False,
                'frequency': 'daily', # '2dayOfMonth,Friday'
                'delay': 30
            },
            'groups': {
                'platform': 'gc',
                'report_name': 'groups',
                'upload_table': 'groups',
                'func_name': self.get_groups,
                'uniq_columns': 'id',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': False,
                'frequency': 'daily', # '2dayOfMonth,Friday'
                'delay': 30
            }
        }

    def translate_keys(self, list_of_dicts):
        try:
            def transliterate_key(key):
                tr = translit(key, 'ru', reversed=True)
                return    tr.replace(' ','_').replace('-','_').replace(",",'').replace("'",'').replace(".",'').replace("(",'').replace(")",'').lower()
            for dictionary in list_of_dicts:
                new_dict = {}
                for key, value in dictionary.items():
                    # Транслитерируем ключ
                    english_key = transliterate_key(key)
                    new_dict[english_key] = value
                dictionary.clear()  # Очищаем оригинальный словарь
                dictionary.update(new_dict)  # Обновляем его новыми значениями с английскими ключами
            return list_of_dicts
        except Exception as e:
            message = f'Платформа: GC. Имя: {self.add_name}. Функция: translate_keys. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            raise

    def get_data(self, report):
        try:
            delay = 10
            max_attempts = 10
            querydata = {"key": self.token}
            if report == "groups":
                report += r'/' +str(self.group_id) + r'/users'
            url = f"{self.clientid}/pl/api/account/{report}?created_at[from]={self.start}".replace(r'//pl',r'/pl')
            response = requests.get(url, params=querydata)
            code = response.status_code
            if code == 429:
                self.err429 = True
            if code == 200:
                export_id = response.json()["info"].get("export_id")
                export_url = f"{self.clientid}/pl/api/account/exports/{export_id}".replace(r'//pl',r'/pl')
                for attempt in range(max_attempts):
                    time.sleep(delay)
                    export_response = requests.get(export_url, params=querydata)
                    export_code = export_response.status_code
                    if export_code == 429:
                        self.err429 = True
                    if export_code == 200:
                        json_data = export_response.json()
                        if json_data.get("success"):
                            items = json_data["info"].get("items", [])
                            fields = json_data["info"].get("fields", [])
                            result_data = [{fields[i]: item[i] for i in range(len(fields))} for item in items]
                            return self.translate_keys(result_data)
                    else:
                        message = f"Попытка {attempt + 1}: Статус {str(export_code)}"
                        self.common.log_func(self.bot_token, self.chat_list, message,1)
                return None
            else:
                response.raise_for_status()
        except Exception as e:
            message = f'Платформа: GC. Имя: {self.add_name}. Функция: get_data. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            raise


    def get_users(self, date):
        try:
            final_result = self.get_data('users')
            message = f'Платформа: GC. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_users. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: GC. Имя: {self.add_name}. Даты: {str(date)}. Функция: get_users. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message


    def get_deals(self, date):
        try:
            final_result = self.get_data('deals')
            message = f'Платформа: GC. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_deals. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: GC. Имя: {self.add_name}. Даты: {str(date)}. Функция: get_deals. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message


    def get_payments(self, date):
        try:
            final_result = self.get_data('payments')
            message = f'Платформа: GC. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_payments. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: GC. Имя: {self.add_name}. Даты: {str(date)}. Функция: get_payments. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message


    def get_groups(self, date):
        try:
            final_result = self.get_data('groups')
            message = f'Платформа: GC. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_groups. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: GC. Имя: {self.add_name}. Даты: {str(date)}. Функция: get_groups. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message


    def collecting_manager(self):
        report_list = self.reports.replace(' ', '').lower().split(',')
        for report in report_list:
                self.clickhouse = Clickhouse( self.bot_token, self.chat_list, self.message_type, self.host, self.port, self.username, self.password,
                                             self.database, self.start, self.add_name, self.err429, self.backfill_days, self.platform)
                self.clickhouse.collecting_report(
                    self.source_dict[report]['platform'],
                    self.source_dict[report]['report_name'],
                    self.source_dict[report]['upload_table'],
                    self.source_dict[report]['func_name'],
                    self.source_dict[report]['uniq_columns'],
                    self.source_dict[report]['partitions'],
                    self.source_dict[report]['merge_type'],
                    self.source_dict[report]['refresh_type'],
                    self.source_dict[report]['history'],
                    self.source_dict[report]['frequency'],
                    self.source_dict[report]['delay']
                )





import requests
import time
import json
import os
import clickhouse_connect
import pandas as pd
from datetime import datetime, timedelta
from .common import Common

class WBreklama:
    def __init__(self, bot_token:str, chat_list:str, message_type: str, subd: str, add_name: str, token: str , host: str, port: str, username: str, password: str, database: str, start: str, backfill_days: int):
        self.bot_token = bot_token
        self.chat_list = chat_list
        self.message_type = message_type
        self.token = token
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.database = database
        self.subd = subd
        self.add_name = add_name.replace(' ','').replace('-','_')
        self.now = datetime.now()
        self.today = datetime.now().date()
        self.yesterday = self.today - timedelta(days = 1)
        self.start = start
        self.common = Common(self.bot_token, self.chat_list, self.message_type)
        self.backfill_days = backfill_days
        self.err429 = False
        self.client = clickhouse_connect.get_client(host=host, port=port, username=username, password=password, database=database)

    def ch_insert(self, df, to_table):
        data_tuples = [tuple(x) for x in df.to_numpy()]
        self.client.insert(to_table, data_tuples, column_names=df.columns.tolist())

    def chunk_list(self, lst, chunk_size):
        for i in range(0, len(lst), chunk_size):
            yield lst[i:i + chunk_size]

    def convert_to_timestamp(self, datetime_str):
        # Используем pandas для преобразования строки в timestamp
        timestamp = pd.to_datetime(datetime_str, utc=True).timestamp()
        # Округляем до целого числа для совместимости с ClickHouse DateTime
        return int(timestamp)


    def get_names(self, campaign_list):
        try:
            headers = {"Authorization": self.token}
            url = "https://advert-api.wildberries.ru/adv/v1/promotion/adverts"
            response = requests.post(url, json=campaign_list, headers=headers)
            try:
                result = response.json()
            except:
                result = None
            message = f'Платформа: WB_ADS. Имя: {self.add_name}. Функция: get_names. Код: {str(response.status_code)}'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            if response.status_code != 200:
                message = f'Платформа: WB_ADS. Имя: {self.add_name}. Функция: get_names. Результат: {str(result)}'
                self.common.log_func(self.bot_token, self.chat_list, message, 1)
            if response.status_code == 429:
                self.err429 = True
            if response.status_code == 200 and result != None:
                df = pd.json_normalize(result)
                df['advertId'] = df['advertId'].astype('int64')
                required_columns = ["endTime", "createTime", "changeTime", "startTime", "name",  "dailyBudget", "advertId", "status", "type"]
                df = df[required_columns]
                pd.set_option('display.max_columns', None)
                data_types = df.dtypes.reset_index()
                df['endTime'] = df['endTime'].apply(self.convert_to_timestamp)
                df['createTime'] = df['createTime'].apply(self.convert_to_timestamp)
                df['changeTime'] = df['changeTime'].apply(self.convert_to_timestamp)
                df['startTime'] = df['startTime'].apply(self.convert_to_timestamp)
                df['timeStamp'] = self.now
                self.ch_insert(df, f"wb_ads_campaigns_{self.add_name}")
                return response.status_code
        except Exception as e:
            message = f'Платформа: WB_ADS. Имя: {self.add_name}. Функция: get_names. Ошибка: {e}'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return None

    def get_data(self, body, token):
        try:
            headers = {'Authorization': token, 'Content-Type': 'application/json'}
            url = "https://advert-api.wildberries.ru/adv/v2/fullstats"
            json_data = json.dumps(body)
            response = requests.post(url, headers=headers, data=json_data)
            message = f'Платформа: WB_ADS. Имя: {self.add_name}. Функция: get_data. Код: {str(response.status_code)}'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            try: result = response.json()
            except: result = None
            if response.status_code != 200:
                message = f'Платформа: WB_ADS. Имя: {self.add_name}. Функция: get_data. Результат: {str(result)}'
                self.common.log_func(self.bot_token, self.chat_list, message, 3)
            if response.status_code == 429:
                self.err429 = True
            if response.status_code == 200 and result != None:
                final_df, final_booster_df = self.extract_df(result)
                self.ch_insert(final_df, f"wb_ads_data_{self.add_name}")
                if not final_booster_df.empty:
                    self.ch_insert(final_booster_df, f"wb_ads_booster_{self.add_name}")
            return response.status_code
        except Exception as e:
            message = f'Платформа: WB_ADS. Имя: {self.add_name}. Функция: get_data. Ошибка: {e}'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return None

    def get_campaigns_in_period(self, campaign_list, token, start_date ):
        try:
            end_date = self.yesterday.strftime("%Y-%m-%d")
            headers = {"Authorization": token}
            url = "https://advert-api.wildberries.ru/adv/v1/promotion/adverts"
            response = requests.post(url, json=campaign_list, headers=headers)
            try:
                result = response.json()
            except:
                result = None
            message = f'Платформа: WB_ADS. Имя: {self.add_name}. Функция: get_campaigns_in_period. Код: {str(response.status_code)}'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            if response.status_code != 200:
                message = f'Платформа: WB_ADS. Имя: {self.add_name}. Функция: get_campaigns_in_period. Результат: {str(result)}'
                self.common.log_func(self.bot_token, self.chat_list, message, 1)
            if response.status_code == 200:
                df = pd.json_normalize(result)
                df['advertId'] = df['advertId'].astype('int64')
                required_columns = [  "advertId", "createTime", "endTime",]
                df = df[required_columns]
                pd.set_option('display.max_columns', None)
                df['endTime'] = df['endTime'].str[:10]
                df['createTime'] = df['createTime'].str[:10]
                df_filtered = df[((df['createTime'] <= end_date) & (df['endTime'] >= start_date))
                               | ((df['endTime'] >= start_date) & (df['createTime'] <= end_date))
                               | ((df['createTime'] >= start_date) & (df['endTime'] <= end_date))
                               | ((df['createTime'] <= start_date) & (df['endTime'] >= end_date))]
                advert_id_list = df_filtered['advertId'].tolist()

                return advert_id_list
            else:
                return None
        except Exception as e:
            message = f'Платформа: WB_ADS. Имя: {self.add_name}. Функция: get_campaigns_in_period. Ошибка: {e}'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return None


    def create_date_list(self, start_date_str, end_date_str):
        try:
            # Преобразование строк в объекты datetime
            start_date = datetime.strptime(start_date_str, '%Y-%m-%d')
            end_date = datetime.strptime(end_date_str, '%Y-%m-%d')

            # Генерация списка дат
            date_list = []
            current_date = start_date
            while current_date <= end_date:
                date_list.append(current_date.strftime('%Y-%m-%d'))
                current_date += timedelta(days=1)

            return date_list
        except Exception as e:
            message =f'Платформа: WB_ADS. Имя: {self.add_name}. Функция: create_date_list. Ошибка: {e}'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return []

    def extract_df(self,in_json):
        try:
            out_json = []
            out_booster_json = []
            for advert in in_json:
                extract_advert = advert['advertId']
                try:
                    booster_stats = advert['boosterStats']
                    for booster in booster_stats:
                        try:
                            booster_date = booster['date'].replace('Z','')
                            booster_nm = booster['nm']
                            booster_avg = booster['avg_position']
                            out_booster_json.append({
                                                'advertId': extract_advert,
                                                'date': booster_date,
                                                'nm': booster_nm,
                                                'avgPosition': booster_avg     })
                        except:
                            pass
                except:
                    pass
                for day in advert['days']:
                    try:
                        extract_date = day['date']
                        for app in day['apps']:
                            extract_app = app['appType']
                            for nm in app['nm']:
                                extract_nm = nm['nmId']
                                try:
                                    out_json.append({
                                        'advertId': extract_advert,
                                        'date': extract_date,
                                        'appType': extract_app,
                                        'nmId': extract_nm,
                                        'views': nm['views'],
                                        'clicks': nm['clicks'],
                                        'sum': nm['sum'],
                                        'atbs': nm['atbs'],
                                        'orders': nm['orders'],
                                        'shks': nm['shks'],
                                        'sum_price': nm['sum_price'],
                                        'name': nm['name']
                                        })
                                except Exception as e:
                                    message = f"Строка nm: {nm}. Не найдено: {e}"
                                    self.common.log_func(self.bot_token, self.chat_list, message, 1)
                    except Exception as e:
                        message = f'Платформа: WB_ADS. Имя: {self.add_name}. Функция: extract_df. Ошибка распознавания {e}: {str(day)[:1000]}'
                        self.common.log_func(self.bot_token, self.chat_list, message, 3)
            pd.set_option('display.max_columns', None)
            df = pd.DataFrame(out_json)
            booster_df = pd.DataFrame(out_booster_json)
            df['date'] = pd.to_datetime(df['date']).dt.date
            if len(out_booster_json)>0:
                booster_df['date'] = pd.to_datetime(booster_df['date']).dt.date
                booster_df['timeStamp'] = self.now
            df['timeStamp'] = self.now
        except Exception as e:
            message = f'Платформа: WB_ADS. Имя: {self.add_name}. Функция: extract_df. Ошибка: {e}'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
        return df, booster_df

    def wb_reklama_collector(self):
        create_table_query_campaigns = f"""
        CREATE TABLE IF NOT EXISTS wb_ads_campaigns_{self.add_name} (
            createTime DateTime,
            startTime DateTime,
            endTime DateTime,
            changeTime DateTime,
            timeStamp DateTime,
            name String,
            dailyBudget UInt64,
            advertId UInt64,
            status UInt64,
            type UInt64
        ) ENGINE = ReplacingMergeTree(timeStamp)
        ORDER BY advertId
        """

        create_table_query_data = f"""
        CREATE TABLE IF NOT EXISTS wb_ads_data_{self.add_name} (
            date Date,
            advertId UInt64,
            appType UInt64,
            nmId UInt64,
            name String,
            views UInt64,
            clicks UInt64,
            sum Float64,
            atbs UInt64,
            orders UInt64,
            shks UInt64,
            sum_price Float64,
            timeStamp DateTime
        ) ENGINE = ReplacingMergeTree(timeStamp)
        ORDER BY (advertId, date, appType, nmId)
        PARTITION BY date
        """

        create_table_query_booster = f"""
                CREATE TABLE IF NOT EXISTS wb_ads_booster_{self.add_name} (
                    date Date,
                    advertId UInt64,
                    nm UInt64,
                    avgPosition UInt64,
                    timeStamp DateTime
                ) ENGINE = ReplacingMergeTree(timeStamp)
                ORDER BY (advertId, nm, date)
                PARTITION BY date
                """

        create_table_query_collect = f"""
        CREATE TABLE IF NOT EXISTS wb_ads_collection_{self.add_name} (
            date Date,
            advertId UInt64,
            collect Bool
        ) ENGINE = ReplacingMergeTree(collect)
        ORDER BY (advertId, date)
        """

        optimize_data = f"OPTIMIZE TABLE wb_ads_data_{self.add_name} FINAL"
        optimize_booster = f"OPTIMIZE TABLE wb_ads_booster_{self.add_name} FINAL"
        optimize_campaigns = f"OPTIMIZE TABLE wb_ads_campaigns_{self.add_name} FINAL"
        optimize_collection = f"OPTIMIZE TABLE wb_ads_collection_{self.add_name} FINAL"

        now = datetime.now()
        yesterday = now - timedelta(days=1)
        self.client.command(create_table_query_campaigns)
        self.client.command(create_table_query_data)
        self.client.command(create_table_query_booster)
        self.client.command(create_table_query_collect)
        headers = {"Authorization": self.token}
        url = "https://advert-api.wildberries.ru/adv/v1/promotion/count"
        response = requests.get(url, headers=headers)
        if response.status_code != 200:
            message = f'Платформа: WB_ADS. Имя: {self.add_name}. Ошибка получения списка: {response.status_code}'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
        else:
            try:
                result = response.json()
                advert_ids = []
                for advert in result['adverts']:
                    for item in advert['advert_list']:
                        advert_ids.append(item['advertId'])
                for chunk in self.chunk_list(advert_ids, 50):
                    self.get_names(chunk)
                    time.sleep(10)

                active_campaigns = []
                for chunk in self.chunk_list(advert_ids, 50):
                    active_campaigns=active_campaigns + self.get_campaigns_in_period(chunk, self.token, self.start)
                    time.sleep(10)
                message = f'Платформа: WB_ADS. Имя: {self.add_name}. Активные кампании: {str(active_campaigns)}'
                self.common.log_func(self.bot_token, self.chat_list, message, 1)

            # забираем активные из wbcampaigns
                active_campaigns_query = f"""
                        SELECT advertId, createTime, endTime
                        FROM wb_ads_campaigns_{self.add_name}
                        WHERE advertId IN ({', '.join(map(str, active_campaigns))})
                        """
                active_campaigns_query_result = self.client.query(active_campaigns_query)
                df_campaigns = pd.DataFrame(active_campaigns_query_result.result_rows, columns=['advertId', 'createTime', 'endTime'])
                df_campaigns['createTime'] = pd.to_datetime(df_campaigns['createTime']).dt.date
                df_campaigns['endTime'] = pd.to_datetime(df_campaigns['endTime']).dt.date

            # формируем список заданий для wbcollection
                campaigns_date_list = []
                yesterday_date = yesterday.strftime("%Y-%m-%d")
                for _, row in df_campaigns.iterrows():
                    advertId = row['advertId']
                    start_date = row['createTime'].strftime('%Y-%m-%d')
                    end_date = row['endTime'].strftime('%Y-%m-%d')
                    if end_date > yesterday_date:
                        end_date = yesterday_date
                    if start_date < self.start:
                        start_date = self.start
                    date_list = self.create_date_list(start_date, end_date)
                    for date in date_list:
                        campaigns_date_list.append((datetime.strptime(date, '%Y-%m-%d').date(), advertId, False))
                df_active_dates = pd.DataFrame(campaigns_date_list, columns=['date', 'advertId', 'collect'])


            # вставляем задания в wbcollection и делаем оптимайз
                self.ch_insert(df_active_dates, f'wb_ads_collection_{self.add_name}')
                time.sleep(20)

                self.client.command(optimize_collection)
                time.sleep(20)

            # отбираем несделанные даты из wbcollection
                false_dates_query = f"""
                        SELECT distinct date  
                        FROM wb_ads_collection_{self.add_name}
                        WHERE collect = False"""
                collect_days_rows = self.client.query(false_dates_query).result_rows
                collect_days = [item[0] for item in collect_days_rows]
                n_days_ago = now - timedelta(days=self.backfill_days)


            # для каждой даты находим актуальный список кампаний на сбор
                for day in collect_days:
                    if self.err429 == False:
                        difference = n_days_ago.date() - day
                        sql_date = day.strftime('%Y-%m-%d')
                        false_campaigns_by_date_query = f"""
                                SELECT advertId  
                                FROM wb_ads_collection_{self.add_name}
                                WHERE collect = False AND date = '{sql_date}'"""
                        campaigns_to_collect_rows = self.client.query(false_campaigns_by_date_query).result_rows
                        campaigns_to_collect =  list(set([item[0] for item in campaigns_to_collect_rows]))

                # делаем сбор по чанкам для каждой даты
                        for chunk in self.chunk_list(campaigns_to_collect, 50):
                            body = []
                            success_list = []
                            for campaign in chunk:
                                body.append({"id": int(campaign), "dates": [sql_date]})
                                if difference.days >= 0:
                                    success_list.append((day, campaign, True))
                            message = f'Платформа: WB_ADS. Имя: {self.add_name}. Дата: {str(sql_date)}. Кампании: {str(chunk)}'
                            self.common.log_func(self.bot_token, self.chat_list, message, 2)

                # получение данных и вставка в wbdata (единой транзакцией вместе с решением коллекшона)
                            try:
                                wb_json = self.get_data(body, self.token)
                                df_success = pd.DataFrame(success_list, columns=['date', 'advertId', 'collect'])
                                if int(wb_json)==200:
                                    self.ch_insert(df_success, f'wb_ads_collection_{self.add_name}')
                                    message = f'Платформа: WB_ADS. Имя: {self.add_name}. Дата: {str(sql_date)}. Кампании: {str(chunk)}. Результат: ОК'
                                    self.common.log_func(self.bot_token, self.chat_list, message, 2)
                                    self.client.command(optimize_collection)
                            except Exception as e:
                                message = f'Платформа: WB_ADS. Имя: {self.add_name}. Дата: {str(sql_date)}. Ошибка: {str(e)}'
                                self.common.log_func(self.bot_token, self.chat_list, message, 3)
                            time.sleep(61)
            except Exception as e:
                message = f'Платформа: WB_ADS. Имя: {self.add_name}. Функция: wb_reklama. Ошибка: {e}.'
                self.common.log_func(self.bot_token, self.chat_list, message, 3)



        self.client.command(optimize_data)
        time.sleep(20)
        self.client.command(optimize_booster)
        time.sleep(10)
        self.client.command(optimize_campaigns)
        time.sleep(20)




from .common import Common
from .clickhouse import Clickhouse
import requests
from datetime import datetime,timedelta
import clickhouse_connect
import pandas as pd
import os
from dateutil import parser
import time
import hashlib
from io import StringIO
import json


class MRKTbyDate:
    def __init__(self, bot_token:str, chats:str, message_type: str, subd: str,
                 host: str, port: str, username: str, password: str, database: str,
                 add_name: str, clientid:str, token: str ,  start: str, backfill_days: int, reports :str):
        self.bot_token = bot_token
        self.chat_list = chats.replace(' ', '').split(',')
        self.message_type = message_type
        self.common = Common(self.bot_token, self.chat_list, self.message_type)
        self.clientid = clientid
        self.token = token
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.database = database
        self.subd = subd
        self.add_name = self.common.transliterate_key(add_name)
        self.now = datetime.now()
        self.today = datetime.now().date()
        self.start = start
        self.reports = reports
        self.backfill_days = backfill_days
        self.platform = 'wb'

        self.err429 = False
        self.source_dict = {
            'stocks': {
                'platform': 'mrkt',
                'report_name': 'stocks',
                'upload_table': 'stocks',
                'func_name': self.get_all_stocks,
                'uniq_columns': 'warehouseId',
                'partitions': '',
                'merge_type': 'MergeTree',
                'refresh_type': 'delete_all',
                'history': False,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 20
            },
            'mappings': {
                'platform': 'mrkt',
                'report_name': 'mappings',
                'upload_table': 'mappings',
                'func_name': self.fetch_all_offer_mappings,
                'uniq_columns': 'timeStamp',
                'partitions': '',
                'merge_type': 'MergeTree',
                'refresh_type': 'delete_all',
                'history': False,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 20
            },
            'orders': {
                'platform': 'mrkt',
                'report_name': 'orders',
                'upload_table': 'orders',
                'func_name': self.fetch_all_orders,
                'uniq_columns': 'id,creationDate',
                'partitions': 'creationDate',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': True,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 20
            },
        }

    def get_stocks_data(self, campaign_id, token, next_page_token=None):
        try:
            final_list = []
            url = f"https://api.partner.market.yandex.ru/campaigns/{campaign_id}/offers/stocks"
            headers = {
                "Authorization": f"Bearer {token}",
                "Content-Type": "application/json"  # Обязательно для POST-запроса
            }
            payload = {
                "limit": 100
            }
            data = {}
            if next_page_token:
                data = {"page_token": next_page_token}
            response = requests.post(url, headers=headers, json=payload, params=data)
            if response.status_code not in {200, 400, 404}:
                response.raise_for_status()
            result = response.json()
            if "result" not in result:
                return [], None
            warehouses = result["result"].get("warehouses", [])
            for wh in warehouses:
                wh_id = wh.get("warehouseId")
                offers = wh.get("offers", [])

                for o in offers:
                    stock = o.get("stocks", None)
                    updatedAt = o.get("updatedAt", None)
                    if stock != []:
                        for st in stock:
                            start_dict = {'warehouseId': wh_id, 'offerId': o['offerId'], 'updatedAt': updatedAt}
                            final_list.append(start_dict | st)
            next_page_token = result["result"].get("paging", {}).get("nextPageToken")
            return final_list, next_page_token
        except Exception as e:
            message = f'Платформа: MRKT. Имя: {self.add_name}. Даты: {str(date)}. Функция: get_stocks_data. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            raise


    def get_all_stocks(self, date):
        try:
            all_stocks = []
            next_page_token = None
            for _ in range(1000):
                stocks, next_page_token = self.get_stocks_data(self.clientid, self.token, next_page_token)
                all_stocks += stocks
                if not next_page_token:
                    break
                time.sleep(0.2)
            message = f'Платформа: MRKT. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_stocks_data. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return all_stocks
        except Exception as e:
            message = f'Платформа: MRKT. Имя: {self.add_name}. Даты: {str(date)}. Функция: get_stocks_data. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message


    def get_orders_data(self,date, next_page_token=None):
        try:
            final_list = []
            url = f"https://api.partner.market.yandex.ru/campaigns/{self.clientid}/stats/orders"
            headers = {
                "Authorization": f"Bearer {self.token}",
                "Content-Type": "application/json"  # Обязательно для запросов
            }
            payload = {
                'dateFrom': date, 'dateTo': date
            }
            params = {"limit": 200}
            if next_page_token:
                params = {"limit": 200, "page_token": next_page_token}
            response = requests.post(url, headers=headers, json=payload, params=params)
            code = response.status_code
            if code not in {200}:
                response.raise_for_status()
            result = response.json()
            if "result" not in result:
                return [], None
            orders = result["result"].get("orders", [])
            for order in orders:
                final_list.append(order)
            next_page_token = result["result"].get("paging", {}).get("nextPageToken")
            return final_list, next_page_token
        except Exception as e:
            message = f'Платформа: MRKT. Имя: {self.add_name}. Даты: {str(date)}. Функция: get_orders_data. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            raise


    def fetch_all_orders(self, date):
        try:
            all_orders = []
            next_page_token = None
            for _ in range(1000):
                orders, next_page_token = self.get_orders_data(date, next_page_token)
                all_orders += orders
                if not next_page_token:
                    break
                time.sleep(0.1)
            message = f'Платформа: MRKT. Имя: {self.add_name}. Дата: {str(date)}. Функция: fetch_all_orders. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return self.common.spread_table(self.common.spread_table(self.common.spread_table(all_orders)))
        except Exception as e:
            message = f'Платформа: MRKT. Имя: {self.add_name}. Даты: {str(date)}. Функция: fetch_all_orders. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message


    def get_offer_mappings_data(self, next_page_token=None):
        try:
            url = f"https://api.partner.market.yandex.ru/businesses/{self.clientid}/offer-mappings"
            headers = {
                "Authorization": f"Bearer {self.token}",
                "Content-Type": "application/json"
            }
            params = {"limit": 200}
            if next_page_token:
                params["page_token"] = next_page_token
            response = requests.post(url, headers=headers, params=params)
            if response.status_code != 200:
                response.raise_for_status()
            result = response.json()
            if "result" not in result:
                return [], None
            offer_mappings = result["result"].get("offerMappings", [])
            next_page_token = result["result"].get("paging", {}).get("nextPageToken")
            return offer_mappings, next_page_token
        except Exception as e:
            message = f'Платформа: MRKT. Имя: {self.add_name}. Функция: get_offer_mappings_data. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message


    def fetch_all_offer_mappings(self,date):
        try:
            all_offer_mappings = []
            next_page_token = None
            for _ in range(1000):
                offer_mappings, next_page_token = self.get_offer_mappings_data(next_page_token)
                all_offer_mappings.extend(offer_mappings)
                if not next_page_token:
                    break
                time.sleep(0.1)
            message = f'Платформа: MRKT. Имя: {self.add_name}. Дата: {str(date)}. Функция: fetch_all_offer_mappings. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return self.common.spread_table(self.common.spread_table(self.common.spread_table(all_offer_mappings)))
        except Exception as e:
            message = f'Платформа: MRKT. Имя: {self.add_name}. Даты: {str(date)}. Функция: fetch_all_offer_mappings. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message


    # тип отчёта, дата -> данные в CH
    def collecting_manager(self):
        report_list = self.reports.replace(' ', '').lower().split(',')
        for report in report_list:
                self.clickhouse = Clickhouse(self.bot_token, self.chat_list, self.message_type, self.host, self.port, self.username, self.password,
                                             self.database, self.start, self.add_name, self.err429, self.backfill_days, self.platform)
                self.clickhouse.collecting_report(
                    self.source_dict[report]['platform'],
                    self.source_dict[report]['report_name'],
                    self.source_dict[report]['upload_table'],
                    self.source_dict[report]['func_name'],
                    self.source_dict[report]['uniq_columns'],
                    self.source_dict[report]['partitions'],
                    self.source_dict[report]['merge_type'],
                    self.source_dict[report]['refresh_type'],
                    self.source_dict[report]['history'],
                    self.source_dict[report]['frequency'],
                    self.source_dict[report]['delay']
                )












from .common import Common
from .clickhouse import Clickhouse
import requests
from datetime import datetime,timedelta
import clickhouse_connect
import pandas as pd
import os
from dateutil import parser
import time
import hashlib
from io import StringIO
import json
import math


class YMbyDate:
    def __init__(self, bot_token:str, chats:str, message_type: str, subd:              str,
                 host: str, port: str, username: str, password: str, database: str,
                 add_name: str, login: str, token: str ,  start: str, backfill_days: int,
                 report: str, dimensions: str,  metrics: str, filters:str = None):
        self.bot_token = bot_token
        self.chat_list = chats.replace(' ', '').split(',')
        self.message_type = message_type
        self.common =  Common(self.bot_token, self.chat_list, self.message_type)
        self.login = login
        self.token = token
        self.subd = subd
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.database = database
        self.add_name = self.common.transliterate_key(add_name)
        self.now = datetime.now()
        self.today = datetime.now().date()
        self.yesterday = self.today - timedelta(days=1)
        self.start = start
        self.dimensions = dimensions
        self.report = report
        self.metrics = metrics
        self.err429 = False
        self.filters = filters
        self.backfill_days = backfill_days

        self.source_dict = {
            'date': {
                'platform': 'ym',
                'report_name': 'date',
                'upload_table': 'date',
                'func_name': self.get_stat,
                'uniq_columns': 'date',
                'partitions': 'date',
                'merge_type': 'MergeTree',
                'refresh_type': 'delete_date',
                'history': True,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 20
            },
            'nodate': {
                'platform': 'ym',
                'report_name': 'nodate',
                'upload_table': 'nodate',
                'func_name': self.get_data,
                'uniq_columns': 'timeStamp',
                'partitions': '',
                'merge_type': 'MergeTree',
                'refresh_type': 'delete_all',
                'history': False,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 20
            },
        }

    def get_metrika_data(self,date1, date2,limit, offset):
        try:
            url = "https://api-metrika.yandex.ru/stat/v1/data"
            headers = {"Authorization": f"OAuth {self.token}"}
            params = {
                "ids": self.login,
                "metrics": self.metrics,
                "dimensions": self.dimensions,
                "date1": date1,
                "date2": date2,
                "limit": limit,
                'accuracy': 'full'
            }
            if offset != 0 and offset != '0':
                params["offset"] = offset
            if self.filters:
                params["filters"] = self.filters
            response = requests.get(url, headers=headers, params=params)
            if response.status_code != 200:
                response.raise_for_status()
            return response.json()
        except Exception as e:
            message = f'Платформа: YM. Имя: {self.add_name}. Даты: {str(date1)}-{str(date2)}. Функция: get_metrika_data. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            raise

    def fetch_all_metrika_data(self,  date1, date2):
        try:
            all_data = []
            all_data_transformed = []
            dimension_list = self.dimensions.split(',')
            metrics_list = self.metrics.split(',')
            offset = 0
            limit = 10000
            while True:
                data = self.get_metrika_data( date1, date2, limit, offset)
                rows = data.get("data", [])
                all_data.extend(rows)
                if len(rows) < limit:
                    break
                offset += limit
                time.sleep(1)
            for row in all_data:
                transformed_element = {}
                dim_num = 0
                for dim in row['dimensions']:
                    transformed_element[dimension_list[dim_num].split(':')[-1].strip()] = dim['name']
                    dim_num += 1
                met_num = 0
                for met in row['metrics']:
                    transformed_element[metrics_list[met_num].split(':')[-1].strip()] = met
                    met_num += 1
                all_data_transformed.append(transformed_element)
            return all_data_transformed
        except Exception as e:
            message = f'Платформа: YM. Имя: {self.add_name}. Даты: {str(date1)}-{str(date2)}. Функция: fetch_all_metrika_data. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            raise

    def get_data(self, date):
        try:
            final_result = self.fetch_all_metrika_data(self.start, self.yesterday.strftime('%Y-%m-%d'))
            message = f'Платформа: YM. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_data. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: YM. Имя: {self.add_name}. Даты: {str(date)}. Функция: get_data. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return  message


    def get_stat(self, date):
        try:
            final_result = self.fetch_all_metrika_data(date, date)
            message = f'Платформа: YM. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_stat. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: YM. Имя: {self.add_name}. Даты: {str(date)}. Функция: get_stat. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return  message


    def collecting_manager(self):
        if self.report == 'nodate':
            self.platform = 'ym_nodate'
        elif self.report == 'date':
            self.platform = 'ym_date'
        self.clickhouse = Clickhouse(self.bot_token, self.chat_list, self.message_type, self.host, self.port, self.username, self.password, self.database,
                                     self.start, self.add_name, self.err429, self.backfill_days, self.platform)
        self.clickhouse.collecting_report(
            self.source_dict[self.report]['platform'],
            self.source_dict[self.report]['report_name'],
            self.source_dict[self.report]['upload_table'],
            self.source_dict[self.report]['func_name'],
            self.source_dict[self.report]['uniq_columns'],
            self.source_dict[self.report]['partitions'],
            self.source_dict[self.report]['merge_type'],
            self.source_dict[self.report]['refresh_type'],
            self.source_dict[self.report]['history'],
            self.source_dict[self.report]['frequency'],
            self.source_dict[self.report]['delay']
        )











from .common import Common
import requests
from datetime import datetime,timedelta
import clickhouse_connect
import pandas as pd
import os
from dateutil import parser
import time
import hashlib
from io import StringIO
import json
import math

class Clickhouse:
    def __init__(self,  bot_token:str, chat_list:str, message_type: str, host: str, port: str, username: str, password: str, database: str, start:str, add_name:str, err429:bool, backfill_days:int, platform:str):
        self.bot_token = bot_token
        self.chat_list = chat_list
        self.message_type = message_type
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.database = database
        self.now = datetime.now()
        self.start = start
        self.add_name = add_name
        self.err429 = err429
        self.backfill_days = backfill_days
        self.today = datetime.now().date()
        self.platform = platform
        self.common = Common(self.bot_token, self.chat_list, self.message_type)



    def test_clickhouse_connection(self):
        try:
            client = clickhouse_connect.get_client(
                host=self.host,
                port=self.port,
                user=self.username,
                password=self.password,
                database=self.database
            )
            client.command('SELECT 1')
            message = f'Платформа: {self.platform}. Имя: {self.add_name}. Подключение к ClickHouse успешно!'
            try:
                self.common.log_func(self.bot_token, self.chat_list, message,1)
            except:
                self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return True
        except Exception as e:
            message = f'Платформа: {self.platform}. Имя: {self.add_name}. Ошибка подключения к ClickHouse: {e}'
            self.common.log_func(self.bot_token, self.chat_list, message,3)
            return False


    def convert_column_to_text(self, client, table_name, column_name, column_type):
        client.command(f"""ALTER TABLE {table_name} ADD COLUMN test1 {column_type};""")
        print(f"Создан столбец test1: {table_name}, {column_name}, {column_type}")
        client.command(f"""ALTER TABLE {table_name} UPDATE test1 = toString({column_name}) WHERE 1;""")
        time.sleep(5)
        client.command(f"OPTIMIZE TABLE {table_name} FINAL")
        time.sleep(5)
        client.command(f"""ALTER TABLE {table_name} DROP COLUMN {column_name};""")
        client.command(f"""ALTER TABLE {table_name} RENAME COLUMN test1 TO {column_name};""")

    def convert_column_to_date(self, client, table_name, column_name):
        client.command(f"""ALTER TABLE {table_name} ADD COLUMN test2 Date;""")
        print(f"Создан столбец test2: {table_name}, {column_name}")
        client.command(f"""ALTER TABLE {table_name} UPDATE test2 = toDate({column_name}) WHERE 1;""")
        time.sleep(5)
        client.command(f"OPTIMIZE TABLE {table_name} FINAL;")
        time.sleep(5)
        client.command(f"""ALTER TABLE {table_name} DROP COLUMN {column_name};""")
        client.command(f"""ALTER TABLE {table_name} RENAME COLUMN test2 TO {column_name};""")

    # датафрейм, название таблицы -> вставка данных
    def ch_insert(self, df, to_table):
        try:
            data_tuples = [tuple(x) for x in df.to_numpy()]
            client = clickhouse_connect.get_client(host=self.host, port=self.port, username=self.username,
                                                   password=self.password, database=self.database)
            client.insert(to_table, data_tuples, column_names=df.columns.tolist())
            message =f'Платформа: {self.platform}. Имя: {self.add_name}. Таблица: {to_table}. Результат: данные вставлены в CH!'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            optimize_table = f"OPTIMIZE TABLE {to_table} FINAL"
            client.command(optimize_table)
        except Exception as e:
            message = f'Платформа: {self.platform}. Имя: {self.add_name}. Таблица: {to_table}. Функция: ch_insert. Ошибка: {e}'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            raise
        finally:
            if client:
                client.close()

    def ch_execute(self, expression):
        client = None
        try:
            disp_exp = expression.strip()[:30] + '...'
            client = clickhouse_connect.get_client(host=self.host, port=self.port, username=self.username, password=self.password, database=self.database)
            client.command(expression)
            message = f'Платформа: {self.platform}. Имя: {self.add_name}. Выражение {disp_exp} выполнено.'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
        except Exception as e:
            message = f'Платформа: {self.platform}. Имя: {self.add_name}. Ошибка выражения {disp_exp}: {e}'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
        finally:
            if client:
                client.close()

    def ch_check(self, table_name):
        try:
            client = clickhouse_connect.get_client(host=self.host, port=self.port, username=self.username, password=self.password, database=self.database)
            result = client.command(f'EXISTS TABLE {table_name}')
            print(result)
            if result == 1:
                print(f'Таблица {table_name} существует.')
                return True
            else:
                print(f'Таблица {table_name} не существует.')
                return False
        except Exception as e:
            message = f'Платформа: {self.platform}. Имя: {self.add_name}. Таблица: {table_name}. Ошибка: {e}'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
        finally:
            if client:
                client.close()

    def ch_text_columns_set(self, table_name):
        client = clickhouse_connect.get_client(host=self.host, port=self.port, username=self.username,
                                               password=self.password, database=self.database)
        text_columns_set = set()
        try:
            query = f"DESCRIBE TABLE {table_name};"
            result = client.query(query)
            columns_info = result.result_rows
            for col in columns_info:
                elem = f"{col[0]} {col[1]}"
                if 'String' in elem:
                    text_columns_set.add(f"{col[0].strip()}")
        except:
            pass
        return text_columns_set

    # список словарей (данные)+уникальность+имятаблицы -> создание/изменение таблицы ch
    def create_alter_ch(self, data, table_name, uniq_columns, partitions, mergetree):
        try:
            dangerous_columns_set = set(uniq_columns.strip().split(',') + partitions.strip().split(','))
            print(table_name)
            text_columns_set = self.ch_text_columns_set(table_name)
            upload_list = self.common.analyze_column_types(data, uniq_columns, partitions, text_columns_set)
            upload_set = set()
            print('upload_list',upload_list)
            uploads = ''
            for i in upload_list:
                if 'None' not in i:
                    upload_set.add(i)
                    uploads += i + ',\n'
            if partitions == '':
                part_part =''
            else:
                part_part = f'PARTITION BY {partitions}'
            create_table_query_campaigns = f'CREATE TABLE IF NOT EXISTS {table_name} (' + uploads + f'timeStamp DateTime ) ENGINE = {mergetree} ORDER BY ({uniq_columns}) {part_part}'
            # print(create_table_query_campaigns)
            client = clickhouse_connect.get_client(host=self.host, port=self.port, username=self.username, password=self.password, database=self.database)
            client.query(create_table_query_campaigns)
            query = f"DESCRIBE TABLE {table_name};"
            result = client.query(query)
            columns_info = result.result_rows
            current_set = set([f"{col[0]} {col[1]}" for col in columns_info])
            print(current_set)
            current_names_set = set([f"{col[0].strip()}" for col in columns_info])
            diff = list(upload_set - current_set)
            if len(diff) > 0:
                start_alter_exp=f'ALTER TABLE {table_name} '
                for d in diff:
                    column_name = d.split(' ')[0].strip()
                    column_type = d.split(' ')[1].strip()
                    if column_name in current_names_set and 'String' in column_type:
                        message = f'Платформа: {self.platform}. Имя: {self.add_name}. Приведение к тексту {column_name} в {table_name}.'
                        self.common.log_func(self.bot_token, self.chat_list, message, 1)
                        if column_name not in dangerous_columns_set:
                            self.convert_column_to_text(client, table_name, column_name, column_type)
                            alter_exp = f"преобразуем столбец {column_name} в текст"
                    elif column_name in current_names_set and column_type == 'Date':
                        message = f'Платформа: {self.platform}. Имя: {self.add_name}. Приведение к дате {column_name} в {table_name}.'
                        self.common.log_func(self.bot_token, self.chat_list, message, 1)
                        if column_name not in dangerous_columns_set:
                            self.convert_column_to_date(client, table_name, column_name)
                            alter_exp = f"преобразуем столбец {column_name} в дату"
                    else:
                        alter_exp =start_alter_exp + 'ADD COLUMN IF NOT EXISTS ' + d + ' AFTER timeStamp;'
                        message = f'Платформа: {self.platform}. Имя: {self.add_name}. Попытка изменения {table_name}. Формула: {alter_exp}'
                        self.common.log_func(self.bot_token, self.chat_list, message, 1)
                        client.query(alter_exp)
                    message = f'Платформа: {self.platform}. Имя: {self.add_name}. Успешное изменение {table_name}. Формула: {alter_exp}'
                    self.common.log_func(self.bot_token, self.chat_list, message, 2)
                    time.sleep(2)
            else:
                message = f'Платформа: {self.platform}. Имя: {self.add_name}. Данные готовы для вставки в {table_name}'
                self.common.log_func(self.bot_token, self.chat_list, message, 1)
        except Exception as e:
            message = f'Платформа: {self.platform}. Имя: {self.add_name}. Функция: create_alter_ch. Ошибка подготовки данных: {e}'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)



    def get_missing_dates(self, table_name, report_name, start_date_str, include_today):
        try:
            start_date = datetime.strptime(start_date_str, '%Y-%m-%d').date()
            query = f"""
            SELECT date
            FROM {table_name}
            WHERE report = '{report_name}' and collect = True"""
            client = clickhouse_connect.get_client(host=self.host, port=self.port, username=self.username, password=self.password, database=self.database)
            result = client.query(query)
            existing_dates = {row[0] for row in result.result_rows}
            current_date = start_date
            all_dates = set()
            if include_today:
                while current_date <= self.today:
                    all_dates.add(current_date)
                    current_date += timedelta(days=1)
            else:
                while current_date < self.today:
                    all_dates.add(current_date)
                    current_date += timedelta(days=1)
            missing_dates = sorted(all_dates - existing_dates)
            missing_dates_str = [date.strftime('%Y-%m-%d') for date in missing_dates]
            message = f'Платформа: {self.platform}. Имя: {self.add_name}. Таблица: {table_name}. Старт: {start_date}. Функция: get_missing_dates. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return missing_dates_str
        except Exception as e:
            message = f'Платформа: {self.platform}. Имя: {self.add_name}. Таблица: {table_name}. Функция: get_missing_dates. Ошибка: {e}'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return None



    def upload_data(self, platform, report_name,upload_table, func_name, uniq_columns, partitions, merge_type, refresh_type, history, delay, date):
        try:
            if self.err429 == False:
                n_days_ago = self.today - timedelta(days=self.backfill_days)
                table_name = f'{platform}_{upload_table}_{self.add_name}'

                text_columns_set = self.ch_text_columns_set(table_name)
                if refresh_type == 'delete_date':
                    refresh = f"ALTER TABLE {table_name} DROP PARTITION '{date}';"
                elif refresh_type == 'delete_all':
                    refresh = f"TRUNCATE TABLE {table_name};"
                else:
                    refresh = f"OPTIMIZE TABLE {table_name};"
                print(refresh)
                data = func_name(date)
                if not self.common.is_error(data):
                    collect = True
                    if history and datetime.strptime(date, '%Y-%m-%d').date() >= n_days_ago:
                        collect = False
                    collection_data = pd.DataFrame({'date': pd.to_datetime([date], format='%Y-%m-%d'), 'report': [report_name], 'collect': [collect]})
                    if self.common.is_empty(data):
                        message = f'Платформа: {platform}. Имя: {self.add_name}. Репорт: {report_name}. Дата: {date}. ПУСТОЙ ОТВЕТ!'
                        self.common.log_func(self.bot_token, self.chat_list, message, 2)
                    if not self.common.is_empty(data):
                        self.create_alter_ch(data, table_name, uniq_columns, partitions, merge_type)
                        df = self.common.check_and_convert_types(data, uniq_columns, partitions, text_columns_set)
                    if self.ch_check(table_name):
                        self.ch_execute(refresh)
                    if not self.common.is_empty(data):
                        self.ch_insert(df, table_name)
                    self.ch_insert(collection_data, f'{platform}_collection_{self.add_name}')
                    message = f'Платформа: {platform}. Имя: {self.add_name}. Репорт: {report_name}. Дата: {date}. Данные добавлены!'
                    self.common.log_func(self.bot_token, self.chat_list, message, 2)
                time.sleep(delay)
            else:
                message = f'Платформа: {platform}. Имя: {self.add_name}. Таблица: {report_name}. Функция: upload_data. Ошибка: 429.'
                self.common.log_func(self.bot_token, self.chat_list, message, 3)
                raise ValueError("Обнаружена ошибка 429")
        except Exception as e:
            message = f'Платформа: {platform}. Имя: {self.add_name}. Репорт: {report_name}. Дата: {date}. Ошибка вставки: {e}'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            time.sleep(delay)



    def collecting_report(self, platform, report_name, upload_table, func_name, uniq_columns, partitions, merge_type, refresh_type, history, frequency, delay):
        try:
            self.test_clickhouse_connection()
            create_table_query_collect = f"""
                CREATE TABLE IF NOT EXISTS {platform}_collection_{self.add_name} (
                date Date, report String, collect Bool ) ENGINE = ReplacingMergeTree(collect) ORDER BY (report, date)"""
            optimize_collection = f"OPTIMIZE TABLE {platform}_collection_{self.add_name} FINAL"
            self.ch_execute(create_table_query_collect)
            self.ch_execute(optimize_collection)
            time.sleep(4)
            if history:
                date_list = self.get_missing_dates(f'{platform}_collection_{self.add_name}', report_name, self.start,  False)
                for date in date_list:
                    if self.err429 == False and self.common.to_collect(frequency, date):
                        message = f'Платформа: {platform}. Имя: {self.add_name}. Таблица: {upload_table}. Репорт: {report_name}. Дата: {date}. Начинаем сбор...'
                        self.common.log_func(self.bot_token, self.chat_list, message, 2)
                        self.upload_data(platform, report_name, upload_table, func_name, uniq_columns, partitions, merge_type, refresh_type, history, delay, date)
            else:
                date = self.today.strftime('%Y-%m-%d')
                if self.err429 == False and self.common.to_collect(frequency, date):
                    message = f'Платформа: {platform}. Имя: {self.add_name}. Таблица: {upload_table}. Репорт: {report_name}. Дата: {date}. Начинаем сбор...'
                    self.common.log_func(self.bot_token, self.chat_list, message, 2)
                    self.upload_data(platform, report_name, upload_table, func_name, uniq_columns, partitions, merge_type, refresh_type, history, delay, date)
        except Exception as e:
            message = f'Платформа: {platform}. Имя: {self.add_name}. Репорт: {report_name}. Функция: collecting_report. Ошибка сбора: {e}'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)



from .common import Common
from .clickhouse import Clickhouse
import requests
from datetime import datetime,timedelta
import clickhouse_connect
import pandas as pd
import os
from dateutil import parser
import time
import hashlib
from io import StringIO
import json
import math


class YDbyDate:
    def __init__(self, bot_token:str, chats:str, message_type: str, subd:              str,
                 host: str, port: str, username: str, password: str, database: str,
                 add_name: str, login: str, token: str ,  start: str, backfill_days: int,
                 columns : str,  report: str, goals :str = None, attributions :str = None):
        self.bot_token = bot_token
        self.chat_list = chats.replace(' ', '').split(',')
        self.message_type = message_type
        self.common =  Common(self.bot_token, self.chat_list, self.message_type)
        self.login = login
        self.token = token
        self.subd = subd
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.database = database
        self.add_name = self.common.transliterate_key(add_name)
        self.now = datetime.now()
        self.today = datetime.now().date()
        self.yesterday = self.today - timedelta(days=1)
        self.start = start
        self.columns = columns
        self.report = report
        self.goals = goals
        self.err429 = False
        self.attributions = attributions
        self.backfill_days = backfill_days

        self.source_dict = {
            'date': {
                'platform': 'yd',
                'report_name': 'date',
                'upload_table': 'date',
                'func_name': self.get_stat,
                'uniq_columns': 'Date',
                'partitions': 'Date',
                'merge_type': 'MergeTree',
                'refresh_type': 'delete_date',
                'history': True,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 20
            },
            'nodate': {
                'platform': 'yd',
                'report_name': 'nodate',
                'upload_table': 'nodate',
                'func_name': self.get_data,
                'uniq_columns': 'timeStamp',
                'partitions': '',
                'merge_type': 'MergeTree',
                'refresh_type': 'delete_all',
                'history': False,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 20
            },
            'ads': {
                'platform': 'yd',
                'report_name': 'ads',
                'upload_table': 'ads',
                'func_name': self.collect_campaign_ads,
                'uniq_columns': 'AdId',
                'partitions': '',
                'merge_type': 'ReplacingMergeTree(timeStamp)',
                'refresh_type': 'nothing',
                'history': False,
                'frequency': 'daily',  # '2dayOfMonth,Friday'
                'delay': 20
            }
        }

    def tsv_to_dict(self, response):
        try:
            tsv_data = response.text
            data = StringIO(tsv_data)
            df = pd.read_csv(data, sep='\t')
            list_of_dicts = df.to_dict(orient='records')
            return list_of_dicts
        except Exception as e:
            message = f'Платформа: YD. Имя: {self.add_name}. Функция: tsv_to_dict. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            raise


    # дата+токен -> список словарей с заказами (данные)
    def get_report(self, date1, date2):
        try:
            current_hour = self.now.hour
            report_name = self.common.shorten_text(str(date1)+str(date2) + str(self.today) + str(self.login) + str(self.columns)
                                                   + str(self.goals) + str(self.attributions) + str(current_hour))
            headers = {
                "Authorization": "Bearer " + self.token,
                "Client-Login": self.login,
                "Accept-Language": "ru", "processingMode": "auto", "returnMoneyInMicros": "false",
                "skipReportHeader": "true", "skipColumnHeader": "false", "skipReportSummary": "true"
            }
            dict = {
                "SelectionCriteria": {"DateFrom": date1, "DateTo": date2},
                "FieldNames": self.columns.replace(' ', '').split(','),
                "ReportName": report_name,
                "Page": {"Limit": 5000000},
                "ReportType": "CUSTOM_REPORT", "DateRangeType": "CUSTOM_DATE",
                "Format": "TSV", "IncludeVAT": "YES", "IncludeDiscount": "NO"
            }
            if self.goals != None and self.goals != '':
                goals_list = list(map(int, self.goals.replace(' ', '').split(',')))
                goal_dict = {"Goals": goals_list}
                dict = dict | goal_dict
            if self.attributions != None and self.attributions != '':
                att_dict = {"AttributionModels": self.attributions.replace(' ', '').split(',')}
                dict = dict | att_dict
            data = {"params": dict}
            response = requests.post('https://api.direct.yandex.com/json/v5/reports', headers=headers, json=data)
            start_code = response.status_code
            if start_code == 200:
                final_result= self.tsv_to_dict(response)
            elif start_code == 201:
                for i in range(60):
                    time.sleep(10)
                    response = requests.post('https://api.direct.yandex.com/json/v5/reports', headers=headers, json=data)
                    code = response.status_code
                    if code == 200:
                        final_result = self.tsv_to_dict(response)
                        break
            else:
                response.raise_for_status()
            return final_result
        except Exception as e:
            message = f'Платформа: YD. Имя: {self.add_name}. Даты: {str(date1)}-{str(date2)}. Функция: get_report. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            raise

    def get_data(self, date):
        try:
            final_result = self.get_report(self.start, self.yesterday.strftime('%Y-%m-%d'))
            message = f'Платформа: YD. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_data. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: YD. Имя: {self.add_name}. Даты: {str(date)}. Функция: get_data. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return  message


    def get_stat(self, date):
        try:
            final_result = self.get_report(date, date)
            message = f'Платформа: YD. Имя: {self.add_name}. Дата: {str(date)}. Функция: get_stat. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_result
        except Exception as e:
            message = f'Платформа: YD. Имя: {self.add_name}. Даты: {str(date)}. Функция: get_stat. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return  message


    def get_campaigns(self):
        try:
            campaigns_url = 'https://api.direct.yandex.com/json/v5/campaigns'
            headers = {"Authorization": "Bearer " + self.token,
                "Client-Login": self.login,
                "Accept-Language": "ru",
                "Content-Type": "application/json"}
            data = {"method": "get",
                "params": {
                    "SelectionCriteria": {},
                    "FieldNames": ["Id", "Name"]}}
            jsonData = json.dumps(data, ensure_ascii=False).encode('utf8')
            response = requests.post(campaigns_url, data=jsonData, headers=headers)
            camp_data = response.json()['result']['Campaigns']
            return camp_data
        except Exception as e:
            message = f'Платформа: YD. Имя: {self.add_name}. Функция: get_campaigns. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            raise



    def get_ads(self, campaign_id, offset):
        try:
            ads_url = 'https://api.direct.yandex.com/json/v5/ads'
            headers = {
                "Authorization": "Bearer " + self.token,
                "Client-Login": self.login,
                "Accept-Language": "ru",
                "Content-Type": "application/json",
            }
            body = {"method": "get",
                    "params": {"SelectionCriteria": {"CampaignIds": [int(campaign_id)]},
                    "FieldNames": [ "CampaignId", "Id", "State", "Status"],
                    "TextAdFieldNames":["Title", "Title2" ,"Text", "Href"],
                    "Page": { "Limit": 10000, "Offset": offset }
                    }}
            jsonBody = json.dumps(body, ensure_ascii=False).encode('utf8')
            response = requests.post(ads_url, data=jsonBody, headers=headers)
            ads_data = response.json()
            return ads_data['result']['Ads']
        except Exception as e:
            message = f'Платформа: YD. Имя: {self.add_name}. Функция: get_ads. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            raise



    def collect_campaign_ads(self, date):
        try:
            final_list = []
            datestr = self.today.strftime('%Y-%m-%d')
            campaigns = self.get_campaigns()
            for camp in campaigns:
                camp_id = camp['Id']
                offset = 0
                try:
                    for k in range(10):
                        ads = self.get_ads(camp_id, offset)
                        for row in ads:
                            final_list.append({'Date': datestr,'CampaignName': camp['Name'], 'CampaignId': camp['Id'], 'AdId': row['Id'], 'Title': row['TextAd']['Title'],'Title2': row['TextAd']['Title2'], 'Text': row['TextAd']['Text'], 'Href': row['TextAd']['Href'] })
                        if len(ads)<10000:
                            break
                        offset += 10000
                except:
                    pass
            message = f'Платформа: YD. Имя: {self.add_name}. Дата: {str(date)}. Функция: collect_campaign_ads. Результат: ОК'
            self.common.log_func(self.bot_token, self.chat_list, message, 1)
            return final_list
        except Exception as e:
            message = f'Платформа: YD. Имя: {self.add_name}. Даты: {str(date)}. Функция: collect_campaign_ads. Ошибка: {e}.'
            self.common.log_func(self.bot_token, self.chat_list, message, 3)
            return message



    def collecting_manager(self):
        if self.report == 'ads':
            self.platform = 'yd_ads'
        elif self.report == 'nodate':
            self.platform = 'yd_nodate'
        elif self.report == 'date':
            self.platform = 'yd_date'
        self.clickhouse = Clickhouse(self.bot_token, self.chat_list, self.message_type, self.host, self.port, self.username, self.password, self.database,
                                     self.start, self.add_name, self.err429, self.backfill_days, self.platform)
        self.clickhouse.collecting_report(
            self.source_dict[self.report]['platform'],
            self.source_dict[self.report]['report_name'],
            self.source_dict[self.report]['upload_table'],
            self.source_dict[self.report]['func_name'],
            self.source_dict[self.report]['uniq_columns'],
            self.source_dict[self.report]['partitions'],
            self.source_dict[self.report]['merge_type'],
            self.source_dict[self.report]['refresh_type'],
            self.source_dict[self.report]['history'],
            self.source_dict[self.report]['frequency'],
            self.source_dict[self.report]['delay']
        )








